/*****************************************************************************
 *
 * NETSAINT.C - NetSaint Main Program Code
 *
 * Program: NetSaint
 * Version: 0.0.4
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * First Written:   01-28-1999 (start of development for version 0.0.1)
 * Last Modified:   09-01-1999
 *
 * Description:
 *
 * NetSaint is a network monitoring tool that will check hosts and services
 * that you specify.  It has the ability to notify contacts via email, pager,
 * or other user-defined methods when a service or host goes down and
 * recovers.  Service and host monitoring is done through the use of external
 * plugins which can be developed independently of NetSaint.
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/* Only use these includes which are created by auotconf */
/* Others may be system dependent and should be included through these files */
#include "../common/config.h"
#include "netsaint.h"
#include "../common/common.h"


char		config_file[MAX_FILENAME_LENGTH];
char            hosts_config_file[MAX_FILENAME_LENGTH];
char		log_file[MAX_FILENAME_LENGTH];
char            status_file[MAX_FILENAME_LENGTH];
char            temp_file[MAX_FILENAME_LENGTH];

char            physical_html_path[MAX_FILENAME_LENGTH];
char            url_html_path[MAX_FILENAME_LENGTH];
char            url_images_path[MAX_FILENAME_LENGTH];
char            url_docs_path[MAX_FILENAME_LENGTH];
char            url_cgibin_path[MAX_FILENAME_LENGTH];

char            macro_contact_name[MAX_CONTACTNAME_LENGTH];
char            macro_contact_alias[MAX_CONTACTNAME_LENGTH];
char	        macro_host_name[MAX_HOSTNAME_LENGTH];
char	        macro_host_alias[MAX_HOSTNAME_LENGTH];
char	        macro_host_address[MAX_ADDRESS_LENGTH];
char	        macro_service_description[MAX_SERVICEDESC_LENGTH];
char	        macro_service_state[MAX_STATE_LENGTH];
char	        macro_date_time[MAX_DATETIME_LENGTH];
char	        macro_output[MAX_INFO_LENGTH];
char            macro_contact_email[MAX_EMAIL_LENGTH];
char            macro_contact_pager[MAX_PAGER_LENGTH];
char            macro_summary[MAX_COMMANDLINE_LENGTH];
char            macro_admin_email[MAX_EMAIL_LENGTH];
char            macro_admin_pager[MAX_PAGER_LENGTH];
char            macro_host_state[MAX_STATE_LENGTH];
char            macro_state_type[MAX_STATETYPE_LENGTH];
char            macro_current_service_attempt[MAX_ATTEMPT_LENGTH];
char            macro_current_host_attempt[MAX_ATTEMPT_LENGTH];
char            macro_argv[MAX_COMMAND_ARGUMENTS][MAX_COMMANDLINE_LENGTH];

char            generic_summary[MAX_COMMANDLINE_LENGTH];

char            *my_strtok_buffer=NULL;
char            *original_my_strtok_buffer=NULL;

int		log_level=DEFAULT_LOG_LEVEL;
int		use_syslog=DEFAULT_USE_SYSLOG;
int		syslog_level=DEFAULT_SYSLOG_LEVEL;
int             log_notifications=DEFAULT_NOTIFICATION_LOGGING;
int             auto_generate_html=DEFAULT_AUTO_GENERATE_HTML;
int             log_service_retries=DEFAULT_LOG_SERVICE_RETRIES;
int             log_host_retries=DEFAULT_LOG_HOST_RETRIES;
int             log_event_handlers=DEFAULT_LOG_EVENT_HANDLERS;

int             sleep_time=DEFAULT_SLEEP_TIME;
int             interval_length=DEFAULT_INTERVAL_LENGTH;
int             inter_check_delay=DEFAULT_INTER_CHECK_DELAY;

int             max_child_process_time=DEFAULT_MAX_CHILD_PROCESS_TIME;
int             child_process_timed_out=FALSE;
int             total_orphaned_children=0;
int             zombie_cleanup_interval=DEFAULT_ZOMBIE_CLEANUP_INTERVAL;

int             use_agressive_host_checking=DEFAULT_AGRESSIVE_HOST_CHECKING;

int             sigshutdown=FALSE;
int             sigrestart=FALSE;

int             create_html=FALSE;
int             verify_config=FALSE;

contact		*contact_list=NULL;
contactgroup	*contactgroup_list=NULL;
host		*host_list=NULL;
hostgroup	*hostgroup_list=NULL;
service		*service_list=NULL;
timed_event     *event_list=NULL;
notification    *notification_list=NULL;
command         *command_list=NULL;
timeperiod      *timeperiod_list=NULL;





int main(int argc, char **argv){
	int result;
	int error;
	char buffer[MAX_INPUT_BUFFER];

	printf("\nNetSaint 0.0.4\n");
	printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
	printf("Last Modified: 09-01-1999\n");
	printf("License: GPL\n\n");

	/* make sure command line options are valid */
	error=FALSE;

	if(argc<2||argc>3)error=TRUE;

	if(argc==3){
		create_html=FALSE;
		verify_config=FALSE;

		strcpy(config_file,argv[2]);

		if(!strcmp(argv[1],"-h"))
			create_html=TRUE;
		else if(!strcmp(argv[1],"-v"))
			verify_config=TRUE;
	        }

	else if(error==FALSE)
		strncpy(config_file,argv[1],sizeof(config_file));

	/* if there are no command line options (or if we encountered an error), print usage */
	if(error==TRUE){
		printf("Usage: %s [-h | -v] <main_config_file>\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" -h   Create HTML pages for static information (contacts, hosts, etc.) based\n");
		printf("      on the information in the configuration files.  NetSaint will *not* start\n");
		printf("      monitoring any hosts when this option is supplied.\n");
		printf("\n");
		printf(" -v   Verify information in the configuration files.  NetSaint will *not* start\n");
		printf("      monitoring any hosts when this option is supplied.\n");
		printf("\n");
		printf("Note:\n");
		printf("      In order to start monitoring, you must supply the main configuration file\n");
		printf("      as the only command line option.\n");
		printf("\n");
		return ERROR;
		}


	do{

		/* reset program variables */
		reset_variables();

		/* reset the restart flag */
		sigrestart=FALSE;

		/* read in the configuration files */
		result=read_main_config(config_file);

		/* there were no errors detected when reading the config files... */
		if(result==OK){

			/* if we are just create HTML pages, do that and then exit */
			if(create_html==TRUE)
				create_all_html_pages();

			/* if we are just checking the config... */
			else if(verify_config==TRUE){
				result=pre_flight_check();
				if(result==OK)
					printf("\nThings look okay - No obvious problems were detected during the pre-flight check\n");
				else
					printf("\n****> Whoa! One or more problems were detected while running the pre-flight check <****\n");
				printf("\nDone verifying configuration.\n");
		                }

			/* else start monitoring... */
			else{

				snprintf(buffer,sizeof(buffer),"NetSaint 0.0.4 starting...\n");
				write_to_log(buffer);
				write_to_syslog(buffer);

				/* remove the old service status log if present */
				remove(status_file);

				/* run the pre-flight check to make sure everything looks okay*/
				result=pre_flight_check();

				/* the config data looks okay, so start to monitor... */
				if(result==OK){

				        /* auto generate HTML files with config data if we're supposed to */
					if(auto_generate_html)
						create_all_html_pages();

				        /* handle signals (interrupts) */
					setup_sighandler();

				        /* calculate the inter-check delay to use when initially scheduling services */
					calculate_inter_check_delay();

				        /* initialize the event timing loop */
					init_timing_loop();

				        /* create the initial status log */
					create_initial_status_log();

				        /* start monitoring all services */
					event_execution_loop();
				        }

				/* there was a problem running the pre-flight check */
				else{
					snprintf(buffer,sizeof(buffer),"Error: A problem was detected while running the pre-flight check\n");
					write_to_log(buffer);
					write_to_syslog(buffer);
					snprintf(buffer,sizeof(buffer),"Bailing out due to errors...\n");
					write_to_log(buffer);
					write_to_syslog(buffer);				        
				        }
	                        }
		        }

		/* there was a problem reading the config files */
		else{

			/* if the config filename looks fishy, warn the user */
			if(!strstr(config_file,"netsaint.cfg")){
				printf("\nNote: Make sure you are specifying the name of the *main* configuration file\n");
				printf("      on the command line (usually 'netsaint.cfg').\n\n");
		                }
		        }

		/* clean up after ourselves */
		cleanup();

	        }while(sigrestart==TRUE);

	return OK;
	}



/******************************************************************/
/************** CONFIGURATION INPUT FUNCTIONS *********************/
/******************************************************************/

/* process the main configuration file */
int read_main_config(char *config_file){
	char input[MAX_INPUT_BUFFER];
	char variable[MAX_INPUT_BUFFER];
	char value[MAX_INPUT_BUFFER];
	char *temp;
	FILE *fp;
	int current_line=0;
	int result;

#ifdef DEBUG0
	printf("read_main_config() start\n");
#endif
#ifdef DEBUG1
	printf("\tConfig file: %s\n",config_file);
#endif

	/* open the config file for reading */
	fp=fopen(config_file,"r");
	if(fp==NULL){
		printf("Error: Cannot open main configuration file '%s' for reading!\n",config_file);
		return ERROR;
		}

	/* process all lines in the config file */
	for(current_line=1,fgets(input,MAX_INPUT_BUFFER-1,fp);!feof(fp);current_line++,fgets(input,MAX_INPUT_BUFFER-1,fp)){

		/* skip blank lines and comments */
		if(input[0]=='#' || input[0]=='\x0' || input[0]=='\n' || input[0]=='\r')
			continue;

#ifdef DEBUG1
		printf("\tLine %d Raw Input: %s",current_line,input);
#endif

		strip(input);

#ifdef DEBUG1
		printf("\tLine %d Stripped Input: %s\n",current_line,input);
#endif

		/* get the variable name */
		temp=my_strtok(input,"=");

		/* if there is no variable name, return error */
		if(temp==NULL){

			printf("Error in main configuration file '%s' - Line %d (NULL variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* else the variable is good */
		strncpy(variable,temp,sizeof(variable));

		/* get the value */
		temp=my_strtok(NULL,"=");

		/* if no value exists, return error */
		if(temp==NULL){

			printf("Error in main configuration file '%s' - Line %d (NULL value)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* else the value is good */
		strncpy(value,temp,sizeof(value));
		strip(value);

		/* process the variable/value */

		if(!strcmp(variable,"cfg_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Config filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(hosts_config_file,value,sizeof(hosts_config_file));
			strip(hosts_config_file);

#ifdef DEBUG1
			printf("\t\thosts_config_file set to '%s'\n",hosts_config_file);
#endif

			result=read_config_file(hosts_config_file);
			if(result!=OK){
				printf("An error occurred while processing host config file '%s'\n",hosts_config_file);
				fclose(fp);
				return result;
				}
			}
		else if(!strcmp(variable,"log_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Log filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(log_file,value,sizeof(log_file));
			strip(log_file);

#ifdef DEBUG1
			printf("\t\tlog_file set to '%s'\n",log_file);
#endif
			}
		else if(!strcmp(variable,"temp_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Temp filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(temp_file,value,sizeof(temp_file));
			strip(temp_file);

#ifdef DEBUG1
			printf("\t\ttemp_file set to '%s'\n",temp_file);
#endif
			}
		else if(!strcmp(variable,"status_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Status filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(status_file,value,sizeof(status_file));
			strip(status_file);

#ifdef DEBUG1
			printf("\t\tstatus_file set to '%s'\n",status_file);
#endif
			}
		else if(!strcmp(variable,"generic_summary")){
			if(strlen(value)>MAX_COMMANDLINE_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Generic summary too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(generic_summary,value,sizeof(generic_summary));
			strip(generic_summary);

#ifdef DEBUG1
			printf("\t\tgeneric_summary set to '%s'\n",generic_summary);
#endif
		        }
		else if(!strcmp(variable,"admin_email")){
			if(strlen(value)>MAX_EMAIL_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Email address too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(macro_admin_email,value,sizeof(macro_admin_email));
			strip(macro_admin_email);

#ifdef DEBUG1
			printf("\t\tmacro_admin_email set to '%s'\n",macro_admin_email);
#endif
		        }
		else if(!strcmp(variable,"admin_pager")){
			if(strlen(value)>MAX_PAGER_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Pager number/address too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(macro_admin_pager,value,sizeof(macro_admin_pager));
			strip(macro_admin_pager);

#ifdef DEBUG1
			printf("\t\tmacro_admin_pager set to '%s'\n",macro_admin_pager);
#endif
		        }
		else if(!strcmp(variable,"log_level")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'2'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for log_level)",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			log_level=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_level set to %d\n",log_level);
#endif
			}
		else if(!strcmp(variable,"use_syslog")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for use_syslog)",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			use_syslog=atoi(value);

#ifdef DEBUG1
			printf("\t\tuse_syslog set to %d\n",use_syslog);
#endif
			}
		else if(!strcmp(variable,"syslog_level")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'2'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for syslog_level)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			syslog_level=atoi(value);

#ifdef DEBUG1
			printf("\t\tsyslog_level set to %d\n",syslog_level);
#endif
			}
		else if(!strcmp(variable,"log_notifications")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_notifications)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_notifications=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_notifications set to %d\n",log_notifications);
#endif
		        }
		else if(!strcmp(variable,"log_service_retries")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_service_retries)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_service_retries=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_service_retries set to %d\n",log_service_retries);
#endif
		        }
		else if(!strcmp(variable,"log_host_retries")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_host_retries)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_host_retries=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_host_retries set to %d\n",log_host_retries);
#endif
		        }
		else if(!strcmp(variable,"log_event_handlers")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_event_handlers)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_event_handlers=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_event_handlers set to %d\n",log_event_handlers);
#endif
		        }
		else if(!strcmp(variable,"use_agressive_host_checking")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for use_agressive_host_checking)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			use_agressive_host_checking=atoi(value);

#ifdef DEBUG1
			printf("\t\tuse_agressive_host_checking set to %d\n",log_event_handlers);
#endif
		        }
		else if(!strcmp(variable,"sleep_time")){
			strip(value);
			sleep_time=atoi(value);
			if(sleep_time<1)
				sleep_time=DEFAULT_SLEEP_TIME;

#ifdef DEBUG1
			printf("\t\tsleep_time set to %d\n",sleep_time);
#endif
		        }
		else if(!strcmp(variable,"interval_length")){
			strip(value);
			interval_length=atoi(value);
			if(interval_length<1)
				interval_length=DEFAULT_INTERVAL_LENGTH;

#ifdef DEBUG1
			printf("\t\tinterval_length set to %d\n",interval_length);
#endif
		        }
		else if(!strcmp(variable,"physical_html_path")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Physical HTML path too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(physical_html_path,value,sizeof(physical_html_path));
			strip(physical_html_path);
			
			/* add an ending forward slash if necessary */
			if(physical_html_path[strlen(physical_html_path)-1]!='/' && strlen(physical_html_path)<MAX_FILENAME_LENGTH-1)
				strcat(physical_html_path,"/");

#ifdef DEBUG1
			printf("\t\tphysical_html_path set to %s\n",physical_html_path);
#endif
		        }
		else if(!strcmp(variable,"url_html_path")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (URL HTML path too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(url_html_path,value,sizeof(url_html_path));
			strip(url_html_path);
			
			/* add an ending forward slash if necessary */
			if(url_html_path[strlen(url_html_path)-1]!='/' && strlen(url_html_path)<MAX_FILENAME_LENGTH-1)
				strcat(url_html_path,"/");

			snprintf(url_images_path,sizeof(url_docs_path),"%simages/",url_html_path);
			snprintf(url_docs_path,sizeof(url_docs_path),"%sdocs/",url_html_path);

#ifdef DEBUG1
			printf("\t\turl_html_path set to %s\n",url_html_path);
			printf("\t\turl_images_path set to %s\n",url_images_path);
			printf("\t\turl_docs_path set to %s\n",url_docs_path);
#endif
		        }
		else if(!strcmp(variable,"url_cgibin_path")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (URL CGI-BIN path too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(url_cgibin_path,value,sizeof(url_cgibin_path));
			strip(url_cgibin_path);
			
			/* add an ending forward slash if necessary */
			if(url_cgibin_path[strlen(url_cgibin_path)-1]!='/' && strlen(url_cgibin_path)<MAX_FILENAME_LENGTH-1)
				strcat(url_cgibin_path,"/");

#ifdef DEBUG1
			printf("\t\turl_cgibin_path set to %s\n",url_cgibin_path);
#endif
		        }
		else if(!strcmp(variable,"auto_generate_html")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for auto_generate_html)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			auto_generate_html=atoi(value);

#ifdef DEBUG1
			printf("\t\tauto_generate_html set to %d\n",auto_generate_html);
#endif
		        }

		else{

			printf("Error in main configuration file '%s' - Line %d (Unknown variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		}

	fclose(fp);

	/* make sure a log file has been specified */
	strip(log_file);
	if(!strcmp(log_file,"")){
		printf("Error: Log file is not specified in main config file '%s'!\n",config_file);
		return ERROR;
		}

#ifdef DEBUG0
	printf("read_main_config() end\n");
#endif

	return OK;
	}


/* process the host/contact/command/service configuration file */
int read_config_file(char *config_file){
	char input[MAX_INPUT_BUFFER];
	char variable[MAX_INPUT_BUFFER];
	char value[MAX_INPUT_BUFFER];
	char vartype[MAX_VAR_TYPE];
	char varname[MAX_VAR_NAME];
	FILE *fp;
	int current_line=0;
	int result;

#ifdef DEBUG0
	printf("read_config_file() start\n");
	printf("\tConfig file: %s\n",config_file);
#endif

	fp=fopen(config_file,"r");
	if(fp==NULL){
		printf("Error: Cannot open host configuration file '%s' for reading!\n",config_file);
		return ERROR;
		}

	/* read all lines from the config file */
	for(current_line=1,fgets(input,MAX_INPUT_BUFFER-1,fp);!feof(fp);current_line++,fgets(input,MAX_INPUT_BUFFER-1,fp)){

		/* skip blank lines and comments */
		if(input[0]=='#' || input[0]=='\x0' || input[0]=='\n' || input[0]=='\r')
			continue;

#ifdef DEBUG1
		printf("\tLine %d Raw Input: %s",current_line,input);
#endif

		strip(input);

#ifdef DEBUG1
		printf("\tLine %d Stripped Input: %s\n",current_line,input);
#endif

		result=parse_input(input,variable,value);
		if(result!=OK){
			printf("Error in host configuration file '%s' - Line %d (Could not parse input)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

#ifdef DEBUG1
		printf("\tParsed Input: Variable=%s, Value=%s\n",variable,value);
#endif

		result=parse_variable(variable,vartype,varname,MAX_VAR_TYPE,MAX_VAR_NAME);
		if(result!=OK){
			printf("Error in host configuration file '%s' - Line %d (Could not parse variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

#ifdef DEBUG1
		printf("\tVar Type: %s\n",vartype);
		printf("\tVar Name: %s\n",varname);
#endif

		/* decide how to process the variable ...*/

		/* this is a TIMEPERIOD entry */
		if(!strcmp(vartype,"timeperiod"))
			result=add_timeperiod(varname,value,current_line);

		/* this is a HOST entry */
		else if(!strcmp(vartype,"host"))
			result=add_host(varname,value,current_line);

		/* this is a HOSTGROUP entry */
		else if(!strcmp(vartype,"hostgroup"))
			result=add_hostgroup(varname,value,current_line);

		/* this is a CONTACT entry */
		else if(!strcmp(vartype,"contact"))
			result=add_contact(varname,value,current_line);

		/* this is a CONTACTGROUP entry */
		else if(!strcmp(vartype,"contactgroup"))
			result=add_contactgroup(varname,value,current_line);

		/* this is a SERVICE entry */
		else if(!strcmp(vartype,"service"))
			result=add_service(varname,value,current_line);

		/* this is a COMMAND entry */
		else if(!strcmp(vartype,"command"))
			result=add_command(varname,value,current_line);

		/* else we don't know what the hell it is */
		else{

			printf("Error in host configuration file '%s' - Line %d (Unknown variable type)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* check the result code after attempting to add the host, contact, etc. */
		if(result!=OK){
			fclose(fp);
			return ERROR;
			}
		}

	fclose(fp);

#ifdef DEBUG0
	printf("read_config_file() end\n");
#endif

	return OK;
	}


/* parse the raw input line and get the variable and value */
int parse_input(char *input, char *variable, char *value){
	char *temp;

#ifdef DEBUG0
	printf("parse_input() start\n");
#endif

	/* get the variable */
	temp=my_strtok(input,"=");

	/* if it doesn't exist, return an error */
	if(temp==NULL){

#ifdef DEBUG
		printf("\tVariable is NULL\n");
#endif

		return ERROR;
		}

	/* else the variable is good */
	else strcpy(variable,temp);

	/* get the value */
	/* Note: token changed from "=" to "\x0" to fix problem with equal signs in service check commands (0.0.2p1) */
	temp=my_strtok(NULL,"\x0");

	/* if the value doesn't exist, return an error */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tValue is NULL\n");
#endif

		return ERROR;
		}

	/* else the value is good */
	else strcpy(value,temp);

	strip(variable);
	strip(value);

#ifdef DEBUG0
	printf("parse_input() end\n");
#endif

	return OK;
	}


/* parse the variable name */
int parse_variable(char *varstring, char *vartype, char *varname, int maxtypelength, int maxnamelength){
	char *temp;

#ifdef DEBUG0
	printf("parse_variable() start\n");
#endif

	/* get the variable type */
	temp=my_strtok(varstring,"[");

	/* return error if its empty */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tVariable type is NULL\n");
#endif

		return ERROR;
		}

	/* return error if var type is too long */
	if(strlen(temp) > (maxtypelength-1)){

#ifdef DEBUG2
		printf("\tVariable type '%s' is too long\n",temp);
#endif

		return ERROR;
                }

	/* else its good */
	strcpy(vartype,temp);

	/* now get the name of the variable */
	temp=my_strtok(NULL,"]");

	/* return error if its empty */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tVariable name is NULL\n");
#endif

		return ERROR;
		}

	/* return error if var name is too long */
	if(strlen(temp) > (maxnamelength-1)){

#ifdef DEBUG2
		printf("\tVariable name '%s' is too long\n",temp);
#endif

		return ERROR;
                }

	/* else its good */
	strcpy(varname,temp);

	strip(vartype);
	strip(varname);

#ifdef DEBUG0
	printf("parse_variable() end\n");
#endif

	return OK;
	}


/******************************************************************/
/**************** OBJECT ADDITION FUNCTIONS ***********************/
/******************************************************************/


/* add a new timeperiod to the list in memory */
int add_timeperiod(char *name,char *value,int line){
	timeperiod *new_timeperiod;
	timeperiod *temp_timeperiod;
	timeperiod *last_timeperiod;
	timerange *new_timerange;
	char *day_range_start_buffer;
	char *range_buffer;
	char temp_period_buffer[MAX_TIMEPERIOD_LENGTH];
	char *temp_day_range_ptr;
	char *temp_range_ptr;
	char *temp_time_ptr;
	char *temp_buffer;
	char *temp_time_buffer;
	char *temp_alias;
	int hours;
	int minutes;
	unsigned long range_start_time;
	unsigned long range_end_time;
	int day;

#ifdef DEBUG0
	printf("add_timeperiod() start\n");
#endif

	/* make sure there isn't a timeperiod by this name added already */
	temp_timeperiod=NULL;
	temp_timeperiod=find_timeperiod(name,temp_timeperiod);
	if(temp_timeperiod!=NULL){
		printf("Error: Duplicate instance of timeperiod '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the timeperiod name length */
	if(strlen(name)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Timeperiod name '%s' too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* get the alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: Invalid alias/description for timeperiod - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_alias)>MAX_TIMEPERIODALIAS_LENGTH-1){
		printf("Error: Timeperiod alias too long - Line %d\n",line);
		return ERROR;
		}

	/* get the time period day ranges */
	temp_buffer=my_strtok(NULL,"\n");
	if(temp_buffer==NULL){
		printf("Error: Invalid day ranges specified for timeperiod - Line %d\n",line);
		return ERROR;
	        }
	if(strlen(temp_buffer)>MAX_TIMEPERIOD_LENGTH-1){
		printf("Error: Timeperiod day ranges too long - Line %d\n",line);
		return ERROR;
	        }
	strcpy(temp_period_buffer,temp_buffer);
	strip(temp_period_buffer);

	/* allocate memory for the new timeperiod */
	new_timeperiod=malloc(sizeof(timeperiod));
	if(new_timeperiod==NULL){
		printf("Error: Could not allocate memory for timeperiod - Line %d\n",line);
		return ERROR;
	        }

	strip(name);
	strip(temp_alias);

	strcpy(new_timeperiod->name,name);
	strcpy(new_timeperiod->alias,temp_alias);

	/* initialize the time ranges for each day in the time period to NULL */
	for(day=0;day<7;day++)
		new_timeperiod->days[day]=NULL;

	/* parse time ranges for each day in the time period */
	day=0;
	for(temp_buffer=my_strtok(temp_period_buffer,";");temp_buffer!=NULL;temp_buffer=my_strtok(NULL,";"),day++){

		if(day>=7)
			break;

#ifdef DEBUG1
		printf("\nDay #%d\n",day);
#endif

		/* no time range found for today, range already initialized to NULL, so continue */
		if(!strcmp(temp_buffer,""))
			continue;

		temp_day_range_ptr=temp_buffer;
		for(day_range_start_buffer=my_strsep(&temp_day_range_ptr,",");day_range_start_buffer!=NULL;day_range_start_buffer=my_strsep(&temp_day_range_ptr,",")){

			temp_range_ptr=day_range_start_buffer;
			range_buffer=my_strsep(&temp_range_ptr,"-");
			if(range_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }

			temp_time_ptr=range_buffer;
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			hours=atoi(temp_time_buffer);
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			minutes=atoi(temp_time_buffer);

			/* calculate the range start time in seconds */
			range_start_time=(unsigned long)((minutes*60)+(hours*60*60));
			
			range_buffer=my_strsep(&temp_range_ptr,"-");
			if(range_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }

			temp_time_ptr=range_buffer;
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			hours=atoi(temp_time_buffer);

			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			minutes=atoi(temp_time_buffer);

			/* calculate the range end time in seconds */
			range_end_time=(unsigned long)((minutes*60)+(hours*60*60));

			/* allocate memory for the new time range */
			new_timerange=malloc(sizeof(timerange));
			if(new_timerange==NULL){
				printf("Error: Could not allocate memory for time range - Line %d\n",line);
				return ERROR;
			        }

			new_timerange->range_start=range_start_time;
			new_timerange->range_end=range_end_time;

			/* add the new time range to the head of the range list for this day */
			new_timerange->next=new_timeperiod->days[day];
			new_timeperiod->days[day]=new_timerange;

#ifdef DEBUG1
			printf("\tTime Range: %lu - %lu\n",new_timerange->range_start,new_timerange->range_end);
#endif

		        }
		
	        }

	/* add new timeperiod to command list, sorted by name */
	last_timeperiod=timeperiod_list;
	for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){
		if(strcmp(new_timeperiod->name,temp_timeperiod->name)<0){
			new_timeperiod->next=temp_timeperiod;
			if(temp_timeperiod==timeperiod_list)
				timeperiod_list=new_timeperiod;
			else
				last_timeperiod->next=new_timeperiod;
			break;
		        }
		else
			last_timeperiod=temp_timeperiod;
	        }
	if(timeperiod_list==NULL){
		new_timeperiod->next=NULL;
		timeperiod_list=new_timeperiod;
	        }
	else if(temp_timeperiod==NULL){
		new_timeperiod->next=NULL;
		last_timeperiod->next=new_timeperiod;
	        }
		
#ifdef DEBUG1
	printf("\tName:         %s\n",new_timeperiod->name);
	printf("\tAlias:        %s\n",new_timeperiod->alias);
#endif
#ifdef DEBUG0
	printf("add_timeperiod() end\n");
#endif

	return OK;
        }


/* add a new host to the list in memory */
int add_host(char *name,char *value,int line){
	host *temp_host;
	host *new_host;
	host *last_host;
	char *temp_alias;
        char *temp_address;
	char *temp_host_check;
	char *temp_event_handler;
	char *temp_buffer;
	char *temp_parent_host;
	char *temp_notification_period;
	int temp_max_attempts;
	int temp_notify_up;
	int temp_notify_down;
	int temp_notify_unreachable;
	int notification_interval;

#ifdef DEBUG0
	printf("add_host() start\n");
#endif

	/* make sure there isn't a host by this name already added */
	temp_host=NULL;
	temp_host=find_host(name,temp_host);
	if(temp_host!=NULL){
		printf("Error: Duplicate instance of host name '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check name length ... */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* grab the host alias name */
	temp_alias=my_strtok(value,";");

	/* if no alias, use the host name */
	if(temp_alias==NULL)
		temp_alias=name;

	/* check the alias length */
	if(strlen(temp_alias)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host alias '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
		}

	/* grab the host address */
	temp_address=my_strtok(NULL,";");

	
	/* if no address, return error */
	if(temp_address==NULL){
		printf("Error: No host address specified - Line %d\n",line);
		return ERROR;
		}

	if(strlen(temp_address)>MAX_ADDRESS_LENGTH-1){
		printf("Error: Host address '%s' too long - Line %d\n",temp_address,line);
		return ERROR;
		}

	/* get the parent host (if any) - no parent host means this host is on the local net */
	temp_parent_host=my_strtok(NULL,";");
	if(temp_parent_host==NULL)
		temp_parent_host="";

	/* check the parent host name length */
	if(strlen(temp_parent_host)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Parent host name '%s' too long - Line %d\n",temp_parent_host,line);
		return ERROR;
	        }

	/* grab the host check command */
	temp_host_check=my_strtok(NULL,";");
	
	/* no command means we won't check the host */
	if(temp_host_check==NULL)
		temp_host_check="";

	/* check the length of the host check command */
	if(strlen(temp_host_check)>MAX_COMMANDNAME_LENGTH){
		printf("Error: Host check command '%s' too long - Line %d\n",temp_host_check,line);
		return ERROR;
	        }

	/* get the max number of check attempts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid max_attempts value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_max_attempts=atoi(temp_buffer);
	if(temp_max_attempts<=0){
		printf("Error: Invalid max_attempts value for host - Line %d\n",line);
		return ERROR;
		}

	/* get the notification interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: NULL notification_interval value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	notification_interval=atoi(temp_buffer);
	if(notification_interval<1){
		printf("Error: Invalid notification_interval value for host - Line %d\n",line);
		return ERROR;
		}

	/* get the notification timeperiod name */
	temp_notification_period=my_strtok(NULL,";");
	if(temp_notification_period==NULL)
		temp_notification_period="";
	if(strlen(temp_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Host notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_recovery value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_up=atoi(temp_buffer);
	if(temp_notify_up<0){
		printf("Error: Invalid notify_on_recovery value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_up>1)
		temp_notify_up=1;

	/* get the notifcation on down option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_down value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_down=atoi(temp_buffer);
	if(temp_notify_down<0){
		printf("Error: Invalid notify_on_down value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_down>1)
		temp_notify_down=1;

	/* get the notifcation on unreachable option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_unreachable value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_unreachable=atoi(temp_buffer);
	if(temp_notify_unreachable<0){
		printf("Error: Invalid notify_on_unreachable value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_unreachable>1)
		temp_notify_unreachable=1;

	/* get the command name to run when the host state changes */
	temp_event_handler=my_strtok(NULL,";");
	if(temp_event_handler==NULL)
		temp_event_handler="";
	if(strlen(temp_event_handler)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Host event handler command name line too long - Line %d\n",line);
		return ERROR;
		}

	/* allocate memory for a new host */
	new_host=malloc(sizeof(host));
	if(new_host==NULL){
		printf("Error: Could not allocate memory for host '%s'\n",name);
		return ERROR;
		}


	strip(name);
	strip(temp_alias);
	strip(temp_address);
	strip(temp_parent_host);
	strip(temp_host_check);
	strip(temp_event_handler);
	strip(temp_notification_period);

	strcpy(new_host->name,name);
	strcpy(new_host->alias,temp_alias);
	strcpy(new_host->address,temp_address);
	strcpy(new_host->parent_host,temp_parent_host);
	strcpy(new_host->host_check_command,temp_host_check);
	strcpy(new_host->event_handler,temp_event_handler);
	strcpy(new_host->notification_period,temp_notification_period);
	new_host->status=HOST_UP;
	new_host->last_host_notification=0L;
	new_host->notification_interval=notification_interval;
	new_host->notify_on_recovery=temp_notify_up;
	new_host->notify_on_down=temp_notify_down;
	new_host->notify_on_unreachable=temp_notify_unreachable;
	new_host->last_check=0L;
	new_host->max_attempts=temp_max_attempts;
	new_host->current_attempt=0;

	/* add host to front of list */
	/*temp_host=host_list;
	host_list=new_host;
	host_list->next=temp_host;*/

	/* add new host to host list, sorted by host name */
	last_host=host_list;
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(strcmp(new_host->name,temp_host->name)<0){
			new_host->next=temp_host;
			if(temp_host==host_list)
				host_list=new_host;
			else
				last_host->next=new_host;
			break;
		        }
		else
			last_host=temp_host;
	        }
	if(host_list==NULL){
		new_host->next=NULL;
		host_list=new_host;
	        }
	else if(temp_host==NULL){
		new_host->next=NULL;
		last_host->next=new_host;
	        }
		

#ifdef DEBUG1
	printf("\tHost Name:                %s\n",new_host->name);
	printf("\tHost Alias:               %s\n",new_host->alias);
        printf("\tHost Address:             %s\n",new_host->address);
	printf("\tParent Host:              %s\n",new_host->parent_host);
	printf("\tHost Check Command:       %s\n",new_host->host_check_command);
	printf("\tMax. Check Attempts:      %d\n",new_host->max_attempts);
	printf("\tHost Event Handler:       %s\n",new_host->event_handler);
	printf("\tNotify On Down:           %s\n",(new_host->notify_on_down==1)?"yes":"no");
	printf("\tNotify On Unreachable:    %s\n",(new_host->notify_on_unreachable==1)?"yes":"no");
	printf("\tNotify On Recovery:       %s\n",(new_host->notify_on_recovery==1)?"yes":"no");
	printf("\tNotification Interval:    %d\n",new_host->notification_interval);
	printf("\tNotification Time Period: %s\n",new_host->notification_period);
#endif
#ifdef DEBUG0
	printf("add_host() end\n");
#endif

	return OK;
	}


/* add a new host group to the list in memory */
int add_hostgroup(char *name,char *value,int line){
	hostgroup *temp_hostgroup;
	hostgroup *new_hostgroup;
	hostgroup *last_hostgroup;
	char *temp_contacts;
	char *temp_host;
	char *temp_alias;
	int result;

#ifdef DEBUG0
	printf("add_hostgroup() start\n");
#endif

	/* make sure a hostgroup by this name hasn't been added already */
	temp_hostgroup=NULL;
	temp_hostgroup=find_hostgroup(name,temp_hostgroup);
	if(temp_hostgroup!=NULL){
		printf("Error: Duplicate instance of hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the hostgroup name */
	if(strlen(name)>MAX_HOSTGROUPNAME_LENGTH-1){
		printf("Error: Hostgroup name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the group alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: No alias specified for hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the length of the group alias */
	if(strlen(temp_alias)>MAX_HOSTGROUPALIAS_LENGTH-1){
		printf("Error: Alias name '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
	        }

	/* get the contact group */
	temp_contacts=my_strtok(NULL,";");
	if(temp_contacts==NULL){
		printf("Error: No contact groups specified for hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the contact groups */
	if(strlen(temp_contacts)>MAX_CONTACTGROUPS_LENGTH-1){
		printf("Error: Contact group(s) '%s' too long - Line %d\n",temp_contacts,line);
		return ERROR;
		}


	/* allocate memory */
	new_hostgroup=malloc(sizeof(hostgroup));
	if(new_hostgroup==NULL){
		printf("Error: Could not allocate memory for hostgroup '%s'\n",name);
		return ERROR;
		}

	strip(name);
	strip(temp_contacts);
	strip(temp_alias);

	strcpy(new_hostgroup->group_name,name);
	strcpy(new_hostgroup->alias,temp_alias);
	strcpy(new_hostgroup->contact_groups,temp_contacts);

	new_hostgroup->members=NULL;

	/* add this hostgroup to the beginning of the list */
	/*temp_hostgroup=hostgroup_list;
	hostgroup_list=new_hostgroup;
	hostgroup_list->next=temp_hostgroup;*/


	/* add new hostgroup to hostgroup list, sorted by hostgroup name */
	last_hostgroup=hostgroup_list;
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
		if(strcmp(new_hostgroup->group_name,temp_hostgroup->group_name)<0){
			new_hostgroup->next=temp_hostgroup;
			if(temp_hostgroup==hostgroup_list)
				hostgroup_list=new_hostgroup;
			else
				last_hostgroup->next=new_hostgroup;
			break;
		        }
		else
			last_hostgroup=temp_hostgroup;
	        }
	if(hostgroup_list==NULL){
		new_hostgroup->next=NULL;
		hostgroup_list=new_hostgroup;
	        }
	else if(temp_hostgroup==NULL){
		new_hostgroup->next=NULL;
		last_hostgroup->next=new_hostgroup;
	        }
		
#ifdef DEBUG1
	printf("\tGroup name:     %s\n",new_hostgroup->group_name);
	printf("\tAlias:          %s\n",new_hostgroup->alias);
	printf("\tContact groups: %s\n",new_hostgroup->contact_groups);
#endif


	/* get the hosts */
	temp_host=my_strtok(NULL,",");
	while(temp_host!=NULL){

		/* add the hostgroup member */
		result=add_hostgroupmember(temp_host,new_hostgroup,line);
		if(result!=OK)
			return ERROR;

		temp_host=my_strtok(NULL,",");
		}

#ifdef DEBUG0
	printf("add_hostgroup() end\n");
#endif

	return OK;
	}


/* add a new member to a host group */
int add_hostgroupmember(char *name,hostgroup *hostgroupptr,int line){
	hostgroupmember *new_member;

#ifdef DEBUG0
	printf("add_hostgroupmember() start\n");
#endif

	/* check the length of the host name */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* allocate memory for a new member */
	new_member=malloc(sizeof(hostgroupmember));
	if(new_member==NULL){
		printf("Error: Could not allocate memory for host '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	strcpy(new_member->host_name,name);
	strip(new_member->host_name);
	
	/* add the new member to the head of the member list */
	new_member->next=hostgroupptr->members;
	hostgroupptr->members=new_member;

#ifdef DEBUG0
	printf("add_hostgroupmember() end\n");
#endif

	return OK;
        }


/* add a new contact to the list in memory */
int add_contact(char *name,char *value,int line){
	contact *temp_contact;
	contact *new_contact;
	contact *last_contact;
	char *temp_email;
	char *temp_pager;
	char *temp_alias;
	char *temp_commands;
	char *temp_host_commands;
	char *temp_svc_notification_period;
	char *temp_host_notification_period;
	int temp_notify_service_recovery;
	int temp_notify_service_critical;
	int temp_notify_service_warning;
	int temp_notify_host_recovery;
	int temp_notify_host_down;
	int temp_notify_host_unreachable;
	char *temp_buffer;

#ifdef DEBUG0
	printf("add_contact() start\n");
#endif

	/* make sure there isn't a contact by this name already added */
	temp_contact=NULL;
	temp_contact=find_contact(name,temp_contact);
	if(temp_contact!=NULL){
		printf("Error: Duplicate instance of contact '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check name length ... */
	if(strlen(name)>MAX_CONTACTNAME_LENGTH-1){
		printf("Error: Contact name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the alias */
	temp_alias=my_strtok(value,";");

	/* if no alias, use name */
	if(temp_alias==NULL)
		temp_alias=name;

	/* check alias length ... */
	if(strlen(temp_alias)>MAX_CONTACTALIAS_LENGTH-1){
		printf("Error: Contact alias '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
		}

	/* get the service notification timeperiod name */
	temp_svc_notification_period=my_strtok(NULL,";");
	if(temp_svc_notification_period==NULL)
		temp_svc_notification_period="";
	if(strlen(temp_svc_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Contact service notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the host notification timeperiod name */
	temp_host_notification_period=my_strtok(NULL,";");
	if(temp_host_notification_period==NULL)
		temp_host_notification_period="";
	if(strlen(temp_host_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Contact host notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_recovery=atoi(temp_buffer);
	if(temp_notify_service_recovery<0){
		printf("Error: Invalid notify_service_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_recovery>1)
		temp_notify_service_recovery=1;

	/* get the notifcation on critical option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_critical value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_critical=atoi(temp_buffer);
	if(temp_notify_service_critical<0){
		printf("Error: Invalid notify_service_critical value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_critical>1)
		temp_notify_service_critical=1;

	/* get the notifcation on warning option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_warning value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_warning=atoi(temp_buffer);
	if(temp_notify_service_warning<0){
		printf("Error: Invalid notify_service_warning value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_warning>1)
		temp_notify_service_warning=1;

	/* get the notifcation on recovery option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_recovery=atoi(temp_buffer);
	if(temp_notify_host_recovery<0){
		printf("Error: Invalid notify_host_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_recovery>1)
		temp_notify_host_recovery=1;

	/* get the notifcation on down option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_down value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_down=atoi(temp_buffer);
	if(temp_notify_host_down<0){
		printf("Error: Invalid notify_host_down value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_down>1)
		temp_notify_host_down=1;

	/* get the notifcation on unreachable option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_unreachable value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_unreachable=atoi(temp_buffer);
	if(temp_notify_host_unreachable<0){
		printf("Error: Invalid notify_host_unreachable value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_unreachable>1)
		temp_notify_host_unreachable=1;

	/* get the service notification commands */
	temp_commands=my_strtok(NULL,";");
	
	/* if no command, error */
	if(temp_commands==NULL){
		printf("Error: Contact '%s' has no service notification commands - Line %d\n",name,line);
		return ERROR;
	        }

	/* check service commands length */
	if(strlen(temp_commands)>MAX_CONTACTCOMMANDS_LENGTH-1){
		printf("Error: Service notification commands for contact '%s' is too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* get the host notification commands */
	temp_host_commands=my_strtok(NULL,";");
	
	/* if no command, error */
	if(temp_host_commands==NULL){
		printf("Error: Contact '%s' has no host notification commands - Line %d\n",name,line);
		return ERROR;
	        }

	/* check host commands length */
	if(strlen(temp_host_commands)>MAX_CONTACTCOMMANDS_LENGTH-1){
		printf("Error: Host notification commands for contact '%s' is too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* grab the email address */
	temp_email=my_strtok(NULL,";");

	if(temp_email==NULL)
		temp_email="";

	/* check the email address length */
	if(strlen(temp_email)>MAX_EMAIL_LENGTH-1){
		printf("Error: Email address '%s' too long - Line %d\n",temp_email,line);
		return ERROR;
		}

	/* grab the pager number */
	temp_pager=my_strtok(NULL,";");

	if(temp_pager==NULL)
		temp_pager="";

	if(strlen(temp_pager)>MAX_PAGER_LENGTH-1){
		printf("Error: Pager number '%s' too long - Line %d\n",temp_pager,line);
		return ERROR;
		}

	/* error if the contact has no email AND no pager */
	if(!strcmp(temp_pager,"")&&!strcmp(temp_email,"")){
		printf("Error: Contact '%s' has no contact information - Line %d\n",name,line);
		return ERROR;
	        }

	/* allocate memory for a new contact */
	new_contact=malloc(sizeof(contact));
	if(new_contact==NULL){
		printf("Error: Could not allocate memory for contact '%s'\n",name);
		return ERROR;
		}

	strip(name);
	strip(temp_email);
	strip(temp_pager);
	strip(temp_alias);
	strip(temp_commands);
	strip(temp_host_commands);
	strip(temp_svc_notification_period);
	strip(temp_host_notification_period);

	strcpy(new_contact->name,name);
	strcpy(new_contact->email,temp_email);
	strcpy(new_contact->pager,temp_pager);
	strcpy(new_contact->alias,temp_alias);
	strcpy(new_contact->commands,temp_commands);
	strcpy(new_contact->host_commands,temp_host_commands);
	strcpy(new_contact->service_notification_period,temp_svc_notification_period);
	strcpy(new_contact->host_notification_period,temp_host_notification_period);
	new_contact->notify_on_service_recovery=temp_notify_service_recovery;
	new_contact->notify_on_service_critical=temp_notify_service_critical;
	new_contact->notify_on_service_warning=temp_notify_service_warning;
	new_contact->notify_on_host_recovery=temp_notify_host_recovery;
	new_contact->notify_on_host_down=temp_notify_host_down;
	new_contact->notify_on_host_unreachable=temp_notify_host_unreachable;

	/* add contact to front of list */
	/*temp_contact=contact_list;
	contact_list=new_contact;
	contact_list->next=temp_contact;*/

	/* add new contact to contact list, sorted by contact name */
	last_contact=contact_list;
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){
		if(strcmp(new_contact->name,temp_contact->name)<0){
			new_contact->next=temp_contact;
			if(temp_contact==contact_list)
				contact_list=new_contact;
			else
				last_contact->next=new_contact;
			break;
		        }
		else
			last_contact=temp_contact;
	        }
	if(contact_list==NULL){
		new_contact->next=NULL;
		contact_list=new_contact;
	        }
	else if(temp_contact==NULL){
		new_contact->next=NULL;
		last_contact->next=new_contact;
	        }

		
#ifdef DEBUG1
	printf("\tContact Name:                  %s\n",new_contact->name);
	printf("\tContact Alias:                 %s\n",new_contact->alias);
	printf("\tContact Email Address:         %s\n",new_contact->email);
	printf("\tContact Pager Address/Number:  %s\n",new_contact->pager);
	printf("\tSvc Notification Time Period:  %s\n",new_contact->service_notification_period);
	printf("\tHost Notification Time Period: %s\n",new_contact->host_notification_period);
	printf("\tService Notification Commands: %s\n",new_contact->commands);
	printf("\tHost Notification Commands:    %s\n",new_contact->host_commands);
#endif
#ifdef DEBUG0
	printf("add_contact() end\n");
#endif

	return OK;
	}


/* add a new contact group to the list in memory */
int add_contactgroup(char *name,char *value,int line){
	contactgroup *temp_contactgroup;
	contactgroup *new_contactgroup;
	contactgroup *last_contactgroup;
	char *temp_contact;
	char *temp_alias;
	int result;

#ifdef DEBUG0
	printf("add_contactgroup() start\n");
#endif

	/* make sure there isn't a contactgroup by this name added already */
	temp_contactgroup=NULL;
	temp_contactgroup=find_contactgroup(name,temp_contactgroup);
	if(temp_contactgroup!=NULL){
		printf("Error: Duplicate instance of contactgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the contactgroup name */
	if(strlen(name)>MAX_CONTACTGROUPNAME_LENGTH-1){
		printf("Error: Contactgroup name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the group alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: No alias specified for contact group '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the length of the group alias */
	if(strlen(temp_alias)>MAX_CONTACTGROUPALIAS_LENGTH-1){
		printf("Error: Alias name '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
	        }

	/* allocate memory for a new contactgroup entry */
	new_contactgroup=malloc(sizeof(contactgroup));
	if(new_contactgroup==NULL){
		printf("Error: Could not allocate memory for contactgroup '%s' - Line %d\n",name,line);
		return ERROR;
			}

	strip(name);
	strip(temp_alias);

	strcpy(new_contactgroup->group_name,name);
	strcpy(new_contactgroup->alias,temp_alias);

	new_contactgroup->members=NULL;

	/* add contactgroup to head of list */
	/*temp_contactgroup=contactgroup_list;
	contactgroup_list=new_contactgroup;
	contactgroup_list->next=temp_contactgroup;*/


	/* add new contactgroup to contactgroup list, sorted by contactgroup name */
	last_contactgroup=contactgroup_list;
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
		if(strcmp(new_contactgroup->group_name,temp_contactgroup->group_name)<0){
			new_contactgroup->next=temp_contactgroup;
			if(temp_contactgroup==contactgroup_list)
				contactgroup_list=new_contactgroup;
			else
				last_contactgroup->next=new_contactgroup;
			break;
		        }
		else
			last_contactgroup=temp_contactgroup;
	        }
	if(contactgroup_list==NULL){
		new_contactgroup->next=NULL;
		contactgroup_list=new_contactgroup;
	        }
	else if(temp_contactgroup==NULL){
		new_contactgroup->next=NULL;
		last_contactgroup->next=new_contactgroup;
	        }
		

#ifdef DEBUG1
	printf("\tGroup name:   %s\n",new_contactgroup->group_name);
	printf("\tAlias:        %s\n",new_contactgroup->alias);
#endif

	/* get the first contact */
	temp_contact=my_strtok(NULL,",");
	while(temp_contact!=NULL){

		/* add the contactgroup member */
		result=add_contactgroupmember(temp_contact,new_contactgroup,line);
		if(result!=OK)
			return ERROR;

		/* get the next contact */
		temp_contact=my_strtok(NULL,",");
		}

#ifdef DEBUG0
	printf("add_contactgroup() end\n");
#endif

	return OK;
	}


/* add a new member to a contact group */
int add_contactgroupmember(char *name,contactgroup *contactgroupptr,int line){
	contactgroupmember *new_member;

#ifdef DEBUG0
	printf("add_contactgroupmember() start\n");
#endif

	/* check the length of the contact name */
	if(strlen(name)>MAX_CONTACTNAME_LENGTH-1){
		printf("Error: Contact name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* allocate memory for a new member */
	new_member=malloc(sizeof(contactgroupmember));
	if(new_member==NULL){
		printf("Error: Could not allocate memory for contact '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	strcpy(new_member->contact_name,name);
	strip(new_member->contact_name);
	
	/* add the new member to the head of the member list */
	new_member->next=contactgroupptr->members;
	contactgroupptr->members=new_member;

#ifdef DEBUG0
	printf("add_contactgroupmember() end\n");
#endif

	return OK;
        }


/* add a new service to the list in memory */
int add_service(char *name,char *value,int line){
	service *temp_service;
	service *new_service;
	service *last_service;
	char *temp_command;
	char *temp_event_handler;
	char *temp_description;
	char *temp_buffer;
	char *temp_notification_period;
	char *temp_check_period;
	char *temp_contactgroups;
	int temp_max_attempts;
	int temp_check_interval;
	int temp_retry_interval;
	int temp_notify_up;
	int temp_notify_down;
	int temp_notify_warning;
	int temp_notification_interval;

#ifdef DEBUG0
	printf("add_service() start\n");
#endif

	/* check the host name length */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the service description */
	temp_description=my_strtok(value,";");
	if(temp_description==NULL){
		printf("Error: Service description missing - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_description)>MAX_SERVICEDESC_LENGTH-1){
		printf("Error: Service description too long - Line %d\n",line);
		return ERROR;
		}

	/* make sure there isn't a service by this name added already */
	temp_service=NULL;
	temp_service=find_service(name,temp_description,temp_service);
	if(temp_service!=NULL){
		printf("Error: Duplicate instance of service '%s' for host '%s' - Line %d\n",temp_description,name,line);
		return ERROR;
		}

	/* get the check timeperiod name */
	temp_check_period=my_strtok(NULL,";");
	if(temp_check_period==NULL)
		temp_check_period="";
	if(strlen(temp_check_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Service check timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the max number of tries to check service status */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid max_attempts value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_max_attempts=atoi(temp_buffer);
	if(temp_max_attempts<=0){
		printf("Error: Invalid max_attempts value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the check interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid check_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_check_interval=atoi(temp_buffer);
	if(temp_check_interval<=0){
		printf("Error: Invalid check_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the retry interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid retry_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_retry_interval=atoi(temp_buffer);
	if(temp_retry_interval<=0){
		printf("Error: Invalid retry_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the contact groups to be notified */
	temp_contactgroups=my_strtok(NULL,";");
	if(temp_contactgroups==NULL){
		printf("Error: No contact groups specified for service - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_contactgroups)>MAX_CONTACTGROUPS_LENGTH-1){
		printf("Error: Contact group(s) '%s' too long - Line %d\n",temp_contactgroups,line);
		return ERROR;
		}

	/* get the notifcation interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notification_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notification_interval=atoi(temp_buffer);
	/* if notification interval = 0, never notify a second time about service being down */
	if(temp_notification_interval<0){
		printf("Error: Invalid notification_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the notification timeperiod name */
	temp_notification_period=my_strtok(NULL,";");
	if(temp_notification_period==NULL)
		temp_notification_period="";
	if(strlen(temp_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Service notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_recovery value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_up=atoi(temp_buffer);
	if(temp_notify_up<0){
		printf("Error: Invalid notify_on_recovery value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_up>1)
		temp_notify_up=1;

	/* get the notifcation on critical (down) option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_critical value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_down=atoi(temp_buffer);
	if(temp_notify_down<0){
		printf("Error: Invalid notify_on_critical value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_down>1)
		temp_notify_down=1;

	/* get the notifcation on warning option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_warning value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_warning=atoi(temp_buffer);
	if(temp_notify_warning<0){
		printf("Error: Invalid notify_on_warning value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_warning>1)
		temp_notify_warning=1;

	/* get the command name to run when the service state changes */
	temp_event_handler=my_strtok(NULL,";");
	if(temp_event_handler==NULL)
		temp_event_handler="";
	if(strlen(temp_event_handler)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Service event handler command name line too long - Line %d\n",line);
		return ERROR;
		}

	/* get the command name to run when checking this service */
	temp_command=my_strtok(NULL,";");
	if(temp_command==NULL){
		printf("Error: Command name is NULL - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_command)>MAX_COMMANDLINE_LENGTH-1){
		printf("Error: Command line too long - Line %d\n",line);
		return ERROR;
		}

	/* allocate memory */
	new_service=malloc(sizeof(service));
	if(new_service==NULL){
		printf("Error: Could not allocate memory for service - Line %d\n",line);
		return ERROR;
		}

	strip(name);
	strip(temp_command);
	strip(temp_event_handler);
	strip(temp_description);
	strip(temp_notification_period);
	strip(temp_check_period);
	strip(temp_contactgroups);

	strcpy(new_service->host_name,name);
	strcpy(new_service->description,temp_description);
	strcpy(new_service->command,temp_command);
	strcpy(new_service->event_handler,temp_event_handler);
	strcpy(new_service->notification_period,temp_notification_period);
	strcpy(new_service->check_period,temp_check_period);
	new_service->check_interval=temp_check_interval;
	new_service->retry_interval=temp_retry_interval;
	new_service->max_attempts=temp_max_attempts;
	new_service->current_attempt=1;
	new_service->last_state=STATE_OK;
	new_service->state_type=SOFT_STATE;
	new_service->error_was_logged=FALSE;
	new_service->host_problem_at_last_check=FALSE;
	new_service->next_check=0L;
	new_service->last_check=0L;
	strcpy(new_service->contact_groups,temp_contactgroups);
	new_service->notification_interval=temp_notification_interval;
	new_service->last_notification=0L;
	new_service->notify_on_warning=temp_notify_warning;
	new_service->notify_on_critical=temp_notify_down;
	new_service->notify_on_recovery=temp_notify_up;

	/* add service to head of list */
	/*temp_service=service_list;
	service_list=new_service;
	service_list->next=temp_service;*/

	/* add new service to service list, sorted by host name */
	last_service=service_list;
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
		if(strcmp(new_service->host_name,temp_service->host_name)<0){
			new_service->next=temp_service;
			if(temp_service==service_list)
				service_list=new_service;
			else
				last_service->next=new_service;
			break;
		        }
		else
			last_service=temp_service;
	        }
	if(service_list==NULL){
		new_service->next=NULL;
		service_list=new_service;
	        }
	else if(temp_service==NULL){
		new_service->next=NULL;
		last_service->next=new_service;
	        }
		

#ifdef DEBUG1
	printf("\tHost:                     %s\n",new_service->host_name);
	printf("\tDescription:              %s\n",new_service->description);
	printf("\tCommand:                  %s\n",new_service->command);
	printf("\tCheck Interval:           %d\n",new_service->check_interval);
	printf("\tRetry Interval:           %d\n",new_service->retry_interval);
	printf("\tMax attempts:             %d\n",new_service->max_attempts);
	printf("\tNotification Interval:    %d\n",new_service->notification_interval);
	printf("\tNotification Time Period: %s\n",new_service->notification_period);
	printf("\tNotify On Warning:        %s\n",(new_service->notify_on_warning==1)?"yes":"no");
	printf("\tNotify On Critical:       %s\n",(new_service->notify_on_critical==1)?"yes":"no");
	printf("\tNotify On Recovery:       %s\n",(new_service->notify_on_recovery==1)?"yes":"no");
#endif
#ifdef DEBUG0
	printf("add_service() end\n");
#endif

	return OK;
	}


/* add a new command to the list in memory */
int add_command(char *name,char *value,int line){
	command *new_command;
	command *temp_command;
	command *last_command;

#ifdef DEBUG0
	printf("add_command() start\n");
#endif

	/* make sure there isn't a command by this name added already */
	temp_command=NULL;
	temp_command=find_command(name,temp_command);
	if(temp_command!=NULL){
		printf("Error: Duplicate instance of command '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the command name length */
	if(strlen(name)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Command name '%s' too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the command line length */
	if(strlen(value)>MAX_COMMANDLINE_LENGTH-1){
		printf("Error: Command line '%s' too long - Line %d\n",value,line);
		return ERROR;
	        }

	/* allocate memory for the new command */
	new_command=malloc(sizeof(command));
	if(new_command==NULL){
		printf("Error: Could not allocate memory for command - Line %d\n",line);
		return ERROR;
	        }

	strip(name);
	strip(value);

	strcpy(new_command->name,name);
	strcpy(new_command->command_line,value);

	/* add command to head of list */
	/*temp_command=command_list;
	command_list=new_command;
	command_list->next=temp_command;*/

	/* add new command to command list, sorted by command name */
	last_command=command_list;
	for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){
		if(strcmp(new_command->name,temp_command->name)<0){
			new_command->next=temp_command;
			if(temp_command==command_list)
				command_list=new_command;
			else
				last_command->next=new_command;
			break;
		        }
		else
			last_command=temp_command;
	        }
	if(command_list==NULL){
		new_command->next=NULL;
		command_list=new_command;
	        }
	else if(temp_command==NULL){
		new_command->next=NULL;
		last_command->next=new_command;
	        }
		
#ifdef DEBUG1
	printf("\tName:         %s\n",new_command->name);
	printf("\tCommand Line: %s\n",new_command->command_line);
#endif
#ifdef DEBUG0
	printf("add_command() end\n");
#endif

	return OK;
        }


/* add a new notification to the list in memory */
int add_notification(contact *cntct){
	notification *new_notification;
	notification *temp_notification;

#ifdef DEBUG0
	printf("add_notification() start\n");
#endif
#ifdef DEBUG1
	printf("\tAdd contact '%s'\n",cntct->name);
#endif

	/* don't add anything if this contact is already on the notification list */
	temp_notification=find_notification(cntct->name);
	if(temp_notification!=NULL)
		return OK;

	/* allocate memory for a new contact in the notification list */
	new_notification=malloc(sizeof(notification));
	if(new_notification==NULL)
		return ERROR;

	/* fill in the contact info */
	new_notification->contact=cntct;

	/* add new notification to head of list */
	new_notification->next=notification_list;
	notification_list=new_notification;

#ifdef DEBUG0
	printf("add_notification() end\n");
#endif

	return OK;
        }


/******************************************************************/
/******************** OBJECT SEARCH FUNCTIONS *********************/
/******************************************************************/

/* given a timeperiod name and a starting point, find a timeperiod from the list in memory */
timeperiod * find_timeperiod(char *name,timeperiod *timeptr){
	timeperiod *temp_timeperiod;

#ifdef DEBUG0
	printf("find_timeperiod() start\n");
#endif

	if(timeptr==NULL)
		temp_timeperiod=timeperiod_list;
	else
		temp_timeperiod=timeptr->next;
	while(temp_timeperiod!=NULL){

		/* we found a matching host */
       	        if(!strcmp(temp_timeperiod->name,name))
		       return temp_timeperiod;

		temp_timeperiod=temp_timeperiod->next;
		}

#ifdef DEBUG0
	printf("find_timeperiod() end\n");
#endif

	/* we couldn't find a matching timeperiod */
	return NULL;
	}


/* given a host name and a starting point, find a host from the list in memory */
host * find_host(char *name,host *hostptr){
	host *temp_host;

#ifdef DEBUG0
	printf("find_host() start\n");
#endif

	if(hostptr==NULL)
		temp_host=host_list;
	else
		temp_host=hostptr->next;
	while(temp_host!=NULL){

		/* we found a matching host */
       	        if(!strcmp(temp_host->name,name))
		       return temp_host;

		temp_host=temp_host->next;
		}

#ifdef DEBUG0
	printf("find_host() end\n");
#endif

	/* we couldn't find a matching host */
	return NULL;
	}


/* given a name and starting point, find a hostgroup from the list in memory */
hostgroup * find_hostgroup(char *name,hostgroup *hostgroupptr){
	hostgroup *temp_hostgroup;

#ifdef DEBUG0
	printf("find_hostgroup() start\n");
#endif

	if(hostgroupptr==NULL)
		temp_hostgroup=hostgroup_list;
	else
		temp_hostgroup=hostgroupptr->next;
	while(temp_hostgroup!=NULL){

		/* we found a matching hostgroup */
		if(!strcmp(temp_hostgroup->host_name,name))
			return temp_hostgroup;

		temp_hostgroup=temp_hostgroup->next;
		}

#ifdef DEBUG0
	printf("find_hostgroup() end\n");
#endif

	/* we couldn't find a matching hostgroup */
	return NULL;
	}


/* given a host name and a starting point, find a member of a host group */
hostgroupmember * find_hostgroupmember(char *name,hostgroup *hostgroupptr,hostgroupmember *memberptr){
	hostgroupmember *temp_member;

#ifdef DEBUG0
	printf("find_hostgroupmember() start\n");
#endif

	if(memberptr==NULL)
		temp_member=hostgroupptr->members;
	else
		temp_member=memberptr->next;
	while(temp_member!=NULL){

		/* we found a match */
		if(!strcmp(temp_member->host_name,name))
			return temp_member;

		temp_member=temp_member->next;
	        }
	

#ifdef DEBUG0
	printf("find_hostgroupmember() end\n");
#endif

	/* we couldn't find a matching member */
	return NULL;
        }


/* given a name and a starting point, find a contact from the list in memory */
contact * find_contact(char *name,contact *contactptr){
	contact *temp_contact;

#ifdef DEBUG0
	printf("find_contact() start\n");
#endif

	if(contactptr==NULL)
		temp_contact=contact_list;
	else
		temp_contact=contactptr->next;
	while(temp_contact!=NULL){

		/* we found a matching contact */
		if(!strcmp(temp_contact->name,name))
			return temp_contact;

		temp_contact=temp_contact->next;
		}

#ifdef DEBUG0
	printf("find_contact() end\n");
#endif

	/* we couldn't find a matching contact */
	return NULL;
	}


/* given a group name and a starting point, find a contact group from the list in memory */
contactgroup * find_contactgroup(char *name,contactgroup *contactgroupptr){
	contactgroup *temp_contactgroup;

#ifdef DEBUG0
	printf("find_contactgroup() start\n");
#endif

	if(contactgroupptr==NULL)
		temp_contactgroup=contactgroup_list;
	else
		temp_contactgroup=contactgroupptr->next;
	while(temp_contactgroup!=NULL){

		/* we found a match */
		if(!strcmp(temp_contactgroup->group_name,name))
			return temp_contactgroup;

		temp_contactgroup=temp_contactgroup->next;
		}


#ifdef DEBUG0
	printf("find_contactgroup() end\n");
#endif

	/* we couldn't find a matching contactgroup */
	return NULL;
	}


/* given a contact name and a starting point, find the corresponding member of a contact group */
contactgroupmember * find_contactgroupmember(char *name,contactgroup *contactgroupptr,contactgroupmember *memberptr){
	contactgroupmember *temp_member;

#ifdef DEBUG0
	printf("find_contactgroupmember() start\n");
#endif

	if(memberptr==NULL)
		temp_member=contactgroupptr->members;
	else
		temp_member=memberptr->next;
	while(temp_member!=NULL){

		/* we found a match */
		if(!strcmp(temp_member->contact_name,name))
			return temp_member;

		temp_member=temp_member->next;
	        }
	

#ifdef DEBUG0
	printf("find_contactgroupmember() end\n");
#endif

	/* we couldn't find a matching member */
	return NULL;
        }


/* given a command name, find a command from the list in memory */
command * find_command(char *name,command *commandptr){
	command *temp_command;

#ifdef DEBUG0
	printf("find_command() start\n");
#endif

	if(commandptr==NULL)
		temp_command=command_list;
	else
		temp_command=commandptr->next;
	while(temp_command!=NULL){

		/* we found a match */
		if(!strcmp(name,temp_command->name))
			return temp_command;

		temp_command=temp_command->next;
	        }

#ifdef DEBUG0
	printf("find_command() end\n");
#endif

	/* we couldn't find a matching command */
	return NULL;
        }



/* given a host/service name and a starting point, find a service from the list in memory */
service * find_service(char *host_name,char *svc_desc,service *svcptr){
	service *temp_service;

#ifdef DEBUG0
	printf("find_service() start\n");
#endif

	if(svcptr==NULL)
		temp_service=service_list;
	else
		temp_service=svcptr->next;
	while(temp_service!=NULL){

		/* we found a matching host name and service description */
       	        if(!strcmp(temp_service->host_name,host_name) && !strcmp(temp_service->description,svc_desc))
		       return temp_service;

		temp_service=temp_service->next;
		}

#ifdef DEBUG0
	printf("find_service() end\n");
#endif

	/* we couldn't find a matching service */
	return NULL;
	}



/* given a contact name, find the notification entry for them for the list in memory */
notification * find_notification(char *contact_name){
	notification *temp_notification;

#ifdef DEBUG0
	printf("find_notification() start\n");
#endif
	
	temp_notification=notification_list;
	while(temp_notification!=NULL){
		if(!strcmp(contact_name,temp_notification->contact->name))
		  return temp_notification;
		temp_notification=temp_notification->next;
	        }

#ifdef DEBUG0
	printf("find_notification() end\n");
#endif

	/* we couldn't find the contact in the notification list */
	return NULL;
        }
	


/****************************************************************/
/**************** CONFIG VERIFICATION FUNCTIONS *****************/
/****************************************************************/

/* do a pre-flight check to make sure object relationships make sense */
int pre_flight_check(void){
	contact *temp_contact;
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	host *temp_host;
	hostgroup *temp_hostgroup;
	hostgroupmember *temp_hostgroupmember;
	service *temp_service;
	command *temp_command;
	timeperiod *temp_timeperiod;
	char temp_buffer[MAX_INPUT_BUFFER];
	char *temp_command_name="";
	char *temp_periodname="";
	char contactgroup_names[MAX_CONTACTGROUPS_LENGTH];
	char *current_contactgroup;
	int found;

#ifdef DEBUG0
	printf("pre_flight_check() start\n");
#endif

	/* make sure each contact is part of a contact group */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		/* reset the found flag */
		found=FALSE;

		/* search all the contact groups for this contact */
		for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
			temp_contactgroupmember=find_contactgroupmember(temp_contact->name,temp_contactgroup,NULL);
			if(temp_contactgroupmember!=NULL){
				found=TRUE;
				break;
			        }
		        }

		/* we couldn't find the contact in any contact groups */
		if(found==FALSE){
			sprintf(temp_buffer,"Warning: Contact '%s' is not a member of any contact groups!\n",temp_contact->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }
	        }
	
#ifdef DEBUG0
	printf("\tCompleted contact <member of> contactgroup check\n");
#endif

	/* make sure each contact in contactgroups is valid */
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		/* search all the group members */
		for(temp_contactgroupmember=temp_contactgroup->members;temp_contactgroupmember!=NULL;temp_contactgroupmember=temp_contactgroupmember->next){

			temp_contact=find_contact(temp_contactgroupmember->contact_name,NULL);
			if(temp_contact==NULL){
				sprintf(temp_buffer,"Warning: Contact '%s' specified in contact group '%s' is not defined anywhere!\n",temp_contactgroupmember->contact_name,temp_contactgroup->group_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);			
			        }

		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted contactgroup[contact]=<valid contact> check\n");
#endif

	/* make sure each host has a valid parent host */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* the host is local and does not have a parent host */
		if(!strcmp(temp_host->parent_host,""))
			continue;

		if(find_host(temp_host->parent_host,NULL)==NULL){
			sprintf(temp_buffer,"Error: Host '%s' does not have a valid parent host!\n",temp_host->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted host[parent_host]=<valid host> check\n");
#endif
		

	/* make sure each host is part of a host group */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* reset the found flag */
		found=FALSE;

		/* search all the host groups for this host */
		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
			temp_hostgroupmember=find_hostgroupmember(temp_host->name,temp_hostgroup,NULL);
			if(temp_hostgroupmember!=NULL){
				found=TRUE;
				break;
			        }
		        }

		/* we couldn't find the host in any host groups */
		if(found==FALSE){
			sprintf(temp_buffer,"Warning: Host '%s' is not a member of any host groups!\n",temp_host->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }
	        }
	
#ifdef DEBUG1
	printf("\tCompleted host <-> hostgroup check\n");
#endif

	/* make sure each host defined in a hostgroup actually exists */
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		/* search all the group members */
		for(temp_hostgroupmember=temp_hostgroup->members;temp_hostgroupmember!=NULL;temp_hostgroupmember=temp_hostgroupmember->next){

			temp_host=find_host(temp_hostgroupmember->host_name,NULL);
			if(temp_host==NULL){
				sprintf(temp_buffer,"Warning: Host '%s' specified in host group '%s' is not defined anywhere!\n",temp_hostgroupmember->host_name,temp_hostgroup->group_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);			
			        }

		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted hostgroup[host]=<valid host> check\n");
#endif

	/* make sure each hostgroup has valid contactgroups */
        for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		/* reset the found flag */
		found=FALSE;

		strcpy(contactgroup_names,temp_hostgroup->contact_groups);
		for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
			temp_contactgroup=find_contactgroup(current_contactgroup,NULL);

			if(temp_contactgroup==NULL){
				sprintf(temp_buffer,"Error: Contact group '%s' specified in host group '%s' is not defined anywhere!\n",current_contactgroup,temp_hostgroup->group_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			}
		}

#ifdef DEBUG1
	printf("\tCompleted hostgroup[contactgroups]=<valid contactgroups> check\n");
#endif

	/* make sure each contactgroup is used in at least one hostgroup */
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		/* reset the found flag */
		found=FALSE;

		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
			strcpy(contactgroup_names,temp_hostgroup->contact_groups);
			for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
				if(!strcmp(temp_contactgroup->group_name,current_contactgroup)){
					found=TRUE;
					break;
				        }
			        }
			if(found==TRUE)
				break;
			}
		if(found==FALSE){
			for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
				strcpy(contactgroup_names,temp_service->contact_groups);
				for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
					if(!strcmp(temp_contactgroup->group_name,current_contactgroup)){
						found=TRUE;
						break;
				                }
			                 }
				if(found==TRUE)
					break;
			        }
		        }

		/* we couldn't find a hostgroup or service */
		if(found==FALSE){
			sprintf(temp_buffer,"Warning: Contact group '%s' is not used in any hostgroup or service definition!\n",temp_contactgroup->group_name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			}
		}

#ifdef DEBUG1
	printf("\tCompleted contactgroup <associated with> hostgroup or service check\n");
#endif

	/* make sure each service being monitored has a valid host */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* reset the found flag */
		found=FALSE;

		temp_host=host_list;
		while(temp_host!=NULL){

			/* we found the host! */
			if(!strcmp(temp_service->host_name,temp_host->name)){
				found=TRUE;
				break;
				}

			temp_host=temp_host->next;
			}

		/* we couldn't find an associated host! */
		if(found==FALSE){
			sprintf(temp_buffer,"Error: Host '%s' specified in service '%s' not defined anywhere!\n",temp_service->host_name,temp_service->description);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
			}
		}

#ifdef DEBUG1
	printf("\tCompleted service[host]=<valid host> check\n");
#endif

	/* make sure each service has valid event handler and service check commands */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* check the event handler command */
		temp_command_name=temp_service->event_handler;
		if(strcmp(temp_command_name,"")){
			temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				sprintf(temp_buffer,"Error: Event handler command '%s' specified in service '%s' for host '%s' not defined anywhere\n",temp_command_name,temp_service->description,temp_service->host_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check the service check_command */
		strcpy(temp_buffer,temp_service->command);

		/* raw command lines shouldn't be checked */
		if(temp_buffer[0]=='"')
			continue;

		/* get the command name, leave any arguments behind */
		temp_command_name=my_strtok(temp_buffer,"!");

		temp_command=find_command(temp_command_name,NULL);
		if(temp_command==NULL){
			sprintf(temp_buffer,"Error: Service check command '%s' specified in service '%s' for host '%s' not defined anywhere!\n",temp_command_name,temp_service->description,temp_service->host_name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }			
	        }

#ifdef DEBUG1
	printf("\tCompleted service <has valid> service check and event handler commands check\n");
#endif

	/* make sure each service has valid contactgroups */
        for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* reset the found flag */
		found=FALSE;

		strcpy(contactgroup_names,temp_service->contact_groups);
		for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
			temp_contactgroup=find_contactgroup(current_contactgroup,NULL);

			if(temp_contactgroup==NULL){
				sprintf(temp_buffer,"Error: Contact group '%s' specified in service '%s' for host '%s' is not defined anywhere!\n",current_contactgroup,temp_service->description,temp_service->host_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			}
		}

#ifdef DEBUG1
	printf("\tCompleted service[contactgroups]=<valid contactgroups> check\n");
#endif

	/* make sure each host has at least one service associated with it */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* reset the found flag */
		found=FALSE;

		temp_service=service_list;
		while(temp_service!=NULL){

			/* we found a service! */
			if(!strcmp(temp_host->name,temp_service->host_name)){
				found=TRUE;
				break;
				}

			temp_service=temp_service->next;
			}

		/* we couldn't find a service associated with this host! */
		if(found==FALSE){
			sprintf(temp_buffer,"Warning: Host '%s' has no services associated with it!\n",temp_host->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			}
		}

#ifdef DEBUG1
	printf("\tCompleted host <associated with> service check\n");
#endif


	/* make sure each host has valid host check and event handler commands */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* check the event handler command */
		temp_command_name=temp_host->event_handler;
		if(strcmp(temp_command_name,"")){
			temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				sprintf(temp_buffer,"Error: Event handler command '%s' specified for host '%s' not defined anywhere\n",temp_command_name,temp_host->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check the host check command */
		strcpy(temp_buffer,temp_host->host_check_command);

		/* hosts that don't have check commands defined shouldn't ever be checked... */
		if(!strcmp(temp_buffer,""))
			continue;

		temp_command_name=&temp_buffer[0];
	        temp_command=find_command(temp_command_name,NULL);
		if(temp_command==NULL){
			sprintf(temp_buffer,"Error: Host check command '%s' specified for host '%s' is not defined anywhere!\n",temp_command_name,temp_host->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted host <has valid> host check and event handler commands check\n");
#endif



	/* make sure each contact has valid service notification commands */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		strcpy(temp_buffer,temp_contact->commands);

		if(!strcmp(temp_buffer,"")){
			sprintf(temp_buffer,"Error: Contact '%s' has no service notification commands defined!\n",temp_contact->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }

		temp_command_name=my_strtok(temp_buffer,",");
		while(temp_command_name!=NULL){

		        temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				sprintf(temp_buffer,"Error: Service notification command '%s' specified for contact '%s' is not defined anywhere!\n",temp_command_name,temp_contact->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			temp_command_name=my_strtok(NULL,",");
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted contact <has valid> service notification command(s) check\n");
#endif


	/* make sure each contact has valid host notification commands */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		strcpy(temp_buffer,temp_contact->host_commands);

		if(!strcmp(temp_buffer,"")){
			sprintf(temp_buffer,"Error: Contact '%s' has no host notification commands defined!\n",temp_contact->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }

		temp_command_name=my_strtok(temp_buffer,",");
		while(temp_command_name!=NULL){

			temp_command=NULL;
		        temp_command=find_command(temp_command_name,temp_command);
			if(temp_command==NULL){
				sprintf(temp_buffer,"Error: Host notification command '%s' specified for contact '%s' is not defined anywhere!\n",temp_command_name,temp_contact->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			temp_command_name=my_strtok(NULL,",");
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted contact <has valid> host notification command(s) check\n");
#endif


	/* make sure each contact has a valid service and host notification periods */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		temp_periodname=temp_contact->service_notification_period;

		if(!strcmp(temp_periodname,"")){
			sprintf(temp_buffer,"Warning: Contact '%s' has no service notification time period defined!\n",temp_contact->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				sprintf(temp_buffer,"Error: Service notification period '%s' specified for contact '%s' is not defined anywhere!\n",temp_periodname,temp_contact->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		temp_periodname=temp_contact->host_notification_period;

		if(!strcmp(temp_periodname,"")){
			sprintf(temp_buffer,"Warning: Contact '%s' has no host notification time period defined!\n",temp_contact->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				sprintf(temp_buffer,"Error: Host notification period '%s' specified for contact '%s' is not defined anywhere!\n",temp_periodname,temp_contact->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted contact <has valid> notification time period check\n");
#endif


	/* make sure each host has a valid notification period */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		temp_periodname=temp_host->notification_period;

		if(!strcmp(temp_periodname,"")){
			sprintf(temp_buffer,"Warning: Host '%s' has no notification time period defined!\n",temp_host->name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				sprintf(temp_buffer,"Error: Notification period '%s' specified for host '%s' is not defined anywhere!\n",temp_periodname,temp_host->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted host <has valid> notification time period check\n");
#endif


	/* make sure each service has valid check and notification period */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		temp_periodname=temp_service->check_period;

		if(!strcmp(temp_periodname,"")){
			sprintf(temp_buffer,"Warning: Service '%s' has no check time period defined!\n",temp_service->description);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				sprintf(temp_buffer,"Error: Check period '%s' specified for service '%s' on host '%s' is not defined anywhere!\n",temp_periodname,temp_service->description,temp_service->host_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		temp_periodname=temp_service->notification_period;

		if(!strcmp(temp_periodname,"")){
			sprintf(temp_buffer,"Warning: Service '%s' on host '%s' has no notification time period defined!\n",temp_service->description,temp_service->host_name);
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				sprintf(temp_buffer,"Error: Notification period '%s' specified for service '%s' on host '%s' is not defined anywhere!\n",temp_periodname,temp_service->description,temp_service->host_name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted service <has valid> check and notification time periods check\n");
#endif


#ifdef DEBUG0
	printf("pre_flight_check() end\n");
#endif

	return OK;
	}


/******************************************************************/
/***************** EVENT SCHEDULING FUNCTIONS *********************/
/******************************************************************/

/* calculate the inter check delay to use when initially scheduling service checks */
void calculate_inter_check_delay(void){
	service *temp_service;
	int number_services=0;
	unsigned long check_interval_total=0L;
	double average_check_interval=0.0L;
	double average_inter_check_delay=0.0L;
	int temp_inter_check_delay;

#ifdef DEBUG0
	printf("calculate_inter_check_delay() start\n");
#endif

	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
		number_services++;
		check_interval_total+=temp_service->check_interval;
	        }

	if(number_services==0 || check_interval_total==0)
		return;

	/* adjust the check interval total to correspond to the interval length */
	check_interval_total=(check_interval_total*interval_length)/60L;

	/* calculate the average check interval for services */
	average_check_interval=(double)((double)check_interval_total/(double)number_services);

	/* calculate the average inter check delay (in seconds) needed to evenly space the service checks out */
	average_inter_check_delay=(double)(average_check_interval/(double)number_services);

	temp_inter_check_delay=(int)average_inter_check_delay;

	/* do some bounds checking... */
	if(temp_inter_check_delay<1)
		temp_inter_check_delay=1;

	/* set the global inter check delay value */
	inter_check_delay=temp_inter_check_delay;

#ifdef DEBUG1
	printf("\tTotal service checks:               %d\n",number_services);
	printf("\tCheck interval total:               %lu\n",check_interval_total);
	printf("\tAverage check interval:             %0.2lf sec\n",average_check_interval);
	printf("\tAverage inter-check delay:          %0.2lf sec\n",average_inter_check_delay);
	printf("\tAdjusted inter-check delay:         %d sec\n",inter_check_delay);
#endif
	

#ifdef DEBUG0
	printf("calculate_inter_check_delay() end\n");
#endif

	return;
        }

/* initialize the event timing loop before we start monitoring */
void init_timing_loop(void){
	service *temp_service;
	int current_service;
	time_t current_time;
	timed_event *new_event;

#ifdef DEBUG0
	printf("init_timing_loop() start\n");
#endif

	current_service=0;

	/* add all service checks as separate events */
	temp_service=service_list;
	while(temp_service!=NULL){

		/* get the time right now */
		time(&current_time);

		/* set the next check time for the service */
		temp_service->next_check=current_time+(current_service*inter_check_delay);

		current_service++;

		/* create a new event */
		new_event=malloc(sizeof(timed_event));
		if(new_event!=NULL){
			new_event->event_type=EVENT_SERVICE_CHECK;
			new_event->event_data=(void *)temp_service;
			new_event->run_time=temp_service->next_check;
			schedule_event(new_event);
		        }

		temp_service=temp_service->next;
	        }

	/* add a zombie process cleanup event */
	new_event=malloc(sizeof(timed_event));
	if(new_event!=NULL){
		new_event->event_type=EVENT_ZOMBIE_CLEANUP;
		new_event->event_data=(void *)NULL;
		new_event->run_time=current_time;
		schedule_event(new_event);
	        }

#ifdef DEBUG0
	printf("init_timing_loop() end\n");
#endif

	return;
        }


/* schedule an event in order of execution time */
void schedule_event(timed_event *event){
	timed_event *temp_event;
	service *temp_service;

#ifdef DEBUG0
	printf("schedule_event() start\n");
#endif

	/* if this event is a service check... */
	if(event->event_type==EVENT_SERVICE_CHECK){
		temp_service=(service *)event->event_data;
		event->run_time=temp_service->next_check;
	        }

	/* if this is a zombie cleanup event ... */
	else if(event->event_type==EVENT_ZOMBIE_CLEANUP){
		event->run_time=event->run_time+zombie_cleanup_interval;
	        }

	event->next=NULL;

	/* add the event to the head of the list if there are no other events */
	if(event_list==NULL)
		event_list=event;

        /* add event to head of the list if it should be executed first */
	else if(event->run_time < event_list->run_time){
	        event->next=event_list;
		event_list=event;
	        }

	/* else place the event according to next execution time */
	else{

		temp_event=event_list;
		while(temp_event!=NULL){

			if(temp_event->next==NULL){
				temp_event->next=event;
				break;
			        }
			
			if(event->run_time < temp_event->next->run_time){
				event->next=temp_event->next;
				temp_event->next=event;
				break;
			        }

			temp_event=temp_event->next;
		        }
	        }

#ifdef DEBUG0
	printf("schedule_event() end\n");
#endif

	return;
        }


/******************************************************************/
/********************* MONITORING FUNCTIONS **********************/
/******************************************************************/

/* this is the main event handler loop */
int event_execution_loop(void){
	timed_event *temp_event;
	time_t current_time;
	service *temp_service;

#ifdef DEBUG0
	printf("event_execution_loop() start\n");
#endif

	/* if we don't have any events to handle, exit */
	if(event_list==NULL){
		printf("There aren't any events that need to be handled!\n");
		return OK;
	        }

	while(event_list!=NULL){
	  
		/* remove the first event from the timing loop */
		temp_event=event_list;
		event_list=event_list->next;

		/* get the current time */
		time(&current_time);

#ifdef DEBUG3
		printf("*** Next Event ***\n");
		printf("\tEvent type: %d ",temp_event->event_type);
		if(temp_event->event_type==EVENT_SERVICE_CHECK){
			printf("(service check)\n");
			temp_service=(service *)temp_event->event_data;
			printf("\t\tService Description: %s\n",temp_service->description);
			printf("\t\tAssociated Host:     %s\n",temp_service->host_name);
		        }
		else if(temp_event->event_type==EVENT_ZOMBIE_CLEANUP)
			printf("(zombie cleanup)\n");
		printf("\tEvent time: %s",ctime(&temp_event->run_time));
		printf("\tCurr. time: %s",ctime(&current_time));
#endif
		
		/* wait until its time to run the event */
		while(current_time<temp_event->run_time){

			/* see if we should exit or restart (a signal was encountered) */
			if(sigshutdown==TRUE || sigrestart==TRUE)
				return OK;

		        /* wait for a while */
		        sleep(sleep_time);

		        /* get the time again */
		        time(&current_time);
		        }

		/* see if we should exit or restart (a signal was encountered) */
		if(sigshutdown==TRUE || sigrestart==TRUE)
			return OK;


		/* how should we handle the event? */
		switch(temp_event->event_type){

		case EVENT_SERVICE_CHECK:

			/* monitor a service */
			temp_service=(service *)temp_event->event_data;
			monitor_service(temp_service);
			break;

		case EVENT_ZOMBIE_CLEANUP:

			/* free orphaned child processes */
			clear_zombies();
			break;

		default:

			break;
		        }

		/* reschedule the event */
		schedule_event(temp_event);
	        }

#ifdef DEBUG0
	printf("event_execution_loop() end\n");
#endif

	return OK;
	}


/* this actually does the service monitoring and top-level reaction stuff... */
int monitor_service(service *svc){
	host *temp_host;
	int result;
	int route_result;
	int ignore_service_for_now=FALSE;
	char temp_buffer[MAX_INPUT_BUFFER];
	unsigned long preferred_time=0L;
	unsigned long next_valid_time=0L;
	time_t t;

#ifdef DEBUG0
	printf("monitor_service() start\n");
#endif

	/* make sure can still run this service check (we may have fallen behind in the scheduling queue) */
	time(&t);
	if(!check_time_against_period((unsigned long)t,svc->check_period)){

#ifdef DEBUG3
		printf("\tThe service check fell behind schedule and cannot be checked now...\n");
#endif

		/* get the next valid time we can run the check */
		preferred_time=(unsigned long)t;
		get_next_valid_time(preferred_time,&next_valid_time,svc->check_period);
		svc->next_check=next_valid_time;

		/* NOTE: This was removed 08/03/99 due to an error with the way the log is updated with services
		   that have been skipped */
		/* update the current service status log so CGIs know the correct next check time */
		/*update_status_log_with_service(svc,svc->last_state);*/

#ifdef DEBUG3
		printf("\tThe service check has been rescheduled for %lu (current time is %lu)\n",svc->next_check,t);
#endif

		return OK;
	        }

	/* get the host that this service runs on */
	temp_host=find_host(svc->host_name,NULL);

	/* check the status of the service */
	svc->last_check=time(NULL);
	result=check_service(svc);

#ifdef DEBUG3
	printf("\tService check returned with a result of %d\n",result);
#endif

	/* if there was some error running the command, just skip it (this shouldn't be happening) */
	if(result==ERROR){

		sprintf(temp_buffer,"Warning: There was an error trying to check service '%s' on host '%s'  - it is being delayed for 24 hours\n",svc->description,svc->host_name);
#ifdef DEBUG3
		printf("%s",temp_buffer);
#endif
		write_to_log(temp_buffer);

		/* reschedule the next check for 24 hours later, so it doesn't bother us, but doesn't start a loop */
		svc->next_check=svc->last_check+86400;
		return OK;
	        }

	/* if the state has changed from its previous value, reset the last notification time and run the event handler */
	if(svc->last_state!=result){
		
#ifdef DEBUG3
		printf("\tService '%s' on host '%s' has changed state since last check!\n",svc->description,svc->host_name);
#endif

		/* reset the last notification time */
		svc->last_notification=0L;
	        }

	/* if the service is up and running OK... */
	if(result==STATE_OK){

#ifdef DEBUG3
		printf("\tService '%s' on host '%s' appears to be STATE_OK\n",svc->description,svc->host_name);
#endif

		/* the service check was okay, so the host is up - reverify route to host if host had previous problem */
		if(temp_host->status!=HOST_UP)
			verify_route_to_host(temp_host);

		/* if the host did not have a problem at the last time we checked the service... */
		if(svc->host_problem_at_last_check==FALSE){

			/* the service recovered before the max number of retries was exceeded */
			if(svc->state_type==SOFT_STATE && svc->current_attempt!=1){
			
				strcpy(macro_state_type,"SOFT");
				svc->state_type=SOFT_STATE;

				write_event_to_log(svc,STATE_OK);
				handle_service_event(svc,STATE_OK,SOFT_STATE);
			        }

			/* the service recovered from a real (i.e. "hard") problem */
			else{

				strcpy(macro_state_type,"HARD");
				svc->state_type=HARD_STATE;
		
				if(svc->last_state==STATE_UNKNOWN){

					write_event_to_log(svc,STATE_OK);
					write_event_to_syslog(svc,STATE_OK);
			

					if(svc->notify_on_critical && svc->notify_on_recovery)
						service_notification(svc,STATE_OK);
		                        }
		
				else if(svc->last_state==STATE_WARNING){

					write_event_to_log(svc,STATE_OK);
					write_event_to_syslog(svc,STATE_OK);

					if(svc->notify_on_warning && svc->notify_on_recovery)
						service_notification(svc,STATE_OK);
			                }

				else if(svc->last_state==STATE_CRITICAL){

					write_event_to_log(svc,STATE_OK);
					write_event_to_syslog(svc,STATE_OK);

					if(svc->notify_on_critical && svc->notify_on_recovery)
						service_notification(svc,STATE_OK);
			                }

				/* run the service event handler */
				if(svc->last_state!=STATE_OK)
					handle_service_event(svc,STATE_OK,HARD_STATE);
			        }
			}

		/* clear all variables because we're up and running! */
		svc->last_notification=0L;
		svc->error_was_logged=FALSE;
		svc->host_problem_at_last_check=FALSE;
		svc->last_state=STATE_OK;
		svc->state_type=HARD_STATE;
		svc->current_attempt=1;
		svc->next_check=svc->last_check+(svc->check_interval*interval_length);
		}

	/* hey, something's not working quite like it should... */
	else{

		/* check the route to the host (if necessary) */
		if(temp_host->status==HOST_UP)
			route_result=verify_route_to_host(temp_host);
		else{
			if(use_agressive_host_checking)
				route_result=verify_route_to_host(temp_host);
			else{
				route_result=temp_host->status;
				host_notification(temp_host,temp_host->status);
			        }
		        }

		/* if the host is down or unreachable ... */
		if(route_result!=HOST_UP){

			svc->host_problem_at_last_check=TRUE;
			svc->current_attempt=1;
			ignore_service_for_now=TRUE;
		        }

		/* else the host is up */
		else{

			/*  the host recovered since the last time the service was checked.. */
			if(svc->host_problem_at_last_check==TRUE){

				/* next time the service is checked we shouldn't get into this same case... */
				svc->host_problem_at_last_check=FALSE;

				/* reset the current check counter, so we can mabye avoid a false recovery alarm - added 07/28/99 */
				svc->current_attempt=1;

				/* but ignore the service problem for now, since the host just came back online... */
				ignore_service_for_now=TRUE;
			        }
		        }

		/* if we should retry the service check, do so (except it the host is down or unreachable!) */
		if(svc->current_attempt<svc->max_attempts){

			/* the host is down or unreachable, so don't retry the service check */
			if(temp_host->status!=HOST_UP){

				svc->last_state=result;
				svc->state_type=HARD_STATE;
				svc->error_was_logged=FALSE;
				svc->current_attempt=1;
				svc->next_check=svc->last_check+(svc->check_interval*interval_length);
			        }

			/* host is up, continue to retry the service check */
			else{

				strcpy(macro_state_type,"SOFT");
				svc->state_type=SOFT_STATE;

				write_event_to_log(svc,result);
				write_event_to_syslog(svc,result);

				if(ignore_service_for_now==FALSE){

					/* run the service event handler */
					if(svc->last_state!=result)
						handle_service_event(svc,result,SOFT_STATE);
			                }

				svc->error_was_logged=FALSE;
				svc->current_attempt=svc->current_attempt+1;
				svc->next_check=svc->last_check+(svc->retry_interval*interval_length);
			        }
		        }
			

		/* else we shouldn't retry the check anymore, so handle the error */
		else{

			strcpy(macro_state_type,"HARD");
			svc->state_type=HARD_STATE;

			if(ignore_service_for_now==FALSE){

				if(result==STATE_WARNING){

					write_event_to_log(svc,STATE_WARNING);
					write_event_to_syslog(svc,STATE_WARNING);
					svc->error_was_logged=TRUE;

					if(svc->notify_on_warning==TRUE)
						service_notification(svc,STATE_WARNING);
				        }

				else if(result==STATE_CRITICAL){
					write_event_to_log(svc,STATE_CRITICAL);
					write_event_to_syslog(svc,STATE_CRITICAL);
					svc->error_was_logged=TRUE;

					if(svc->notify_on_critical==TRUE)
						service_notification(svc,STATE_CRITICAL);
				        }

				else if(result==STATE_UNKNOWN){
					write_event_to_log(svc,STATE_UNKNOWN);
					write_event_to_syslog(svc,STATE_UNKNOWN);
					svc->error_was_logged=TRUE;

					if(svc->notify_on_warning==TRUE)
						service_notification(svc,STATE_UNKNOWN);
				        }

				/* run the service event handler */
				if(svc->last_state!=result)
					handle_service_event(svc,result,HARD_STATE);
			        }

			/* set the last state and next check time */
			svc->last_state=result;
			svc->next_check=svc->last_check+(svc->check_interval*interval_length);       
		        }
		}

	/* make sure we rescheduled the check at a valid time */
	preferred_time=svc->next_check;
	get_next_valid_time(preferred_time,&next_valid_time,svc->check_period);
	svc->next_check=next_valid_time;

	/* update the current service status log */
	update_status_log_with_service(svc,result);

#ifdef DEBUG0
	printf("monitor_service() end\n");
#endif

	return OK;
	}


/* this does the actual check of the service */
int check_service(service *svc){
	char raw_command[MAX_COMMANDLINE_LENGTH];
	char processed_command[MAX_COMMANDLINE_LENGTH];
	host *temp_host;
	/*char *buffer;*/
	int result;
	FILE *fp;

#ifdef DEBUG0
	printf("check_service() start\n");
#endif

	/* find the host associated with this service */
	temp_host=NULL;
	temp_host=find_host(svc->host_name,temp_host);

	/* error if we couldn't find the host */
	if(temp_host==NULL){

#ifdef DEBUG2
		printf("\tWarning: No valid host associated with service '%s'!\n",svc->description);
#endif

		return ERROR;
		}

	/* clear all the volatile macros */
	clear_volatile_macros();

	/* grab the macro variables (state is UNKNOWN at this time, along with all contact macros) */
	grab_service_macros(svc,STATE_UNKNOWN);
	grab_host_macros(temp_host);

	/* get the raw command line */
	get_raw_command_line(svc->command,&raw_command[0]);

	/* error if there is no command */
	if(!strcmp(raw_command,"")){

#ifdef DEBUG2
		printf("\tWarning: Raw command line is empty\n");
#endif

		return ERROR;
	        }

	/*strcpy(raw_command,buffer);*/
	strip(raw_command);

#ifdef DEBUG3
	printf("\tRaw Command: %s\n",raw_command);
#endif

	/* process any macros contained in the argument */
	process_macros(raw_command,processed_command);
	strip(processed_command);

	/* redirect stdio to a temp file */
	if(!strcmp(temp_file,""))
	        strcpy(temp_file,DEFAULT_TEMP_FILE);
	strcat(processed_command," >");
	strcat(processed_command,temp_file);

	/* redirect errors to null */
	strcat(processed_command," 2> /dev/null");

#ifdef DEBUG3
	printf("\tProcessed Command: %s\n",processed_command);
#endif

	/* reset the child process timeout flag */
	child_process_timed_out=FALSE;

	/* run the command to check the service... */
	result=my_system(processed_command);

#ifdef DEBUG3
	printf("my_system() returned with result = %d\n",result);
#endif

	/* if the child process timed out, report an error */
	if(child_process_timed_out==TRUE){
		sprintf(macro_output,"Service check timed out after %d seconds\n",max_child_process_time);
	        }

	/* else grab the first line of the program output as the output macro */
	else{
		strcpy(macro_output,"");
		fp=fopen(temp_file,"r");
		if(fp!=NULL){
			fgets(macro_output,MAX_INFO_LENGTH-1,fp);
			macro_output[MAX_INFO_LENGTH-1]='\x0';
			fclose(fp);
	                }
	        }

	/* make sure the output isn't NULL */
	strip(macro_output);
	if(!strcmp(macro_output,""))
		strcpy(macro_output,"(No Information Available)");

	/* remove the temp file */
	remove(temp_file);

	/* return an unknown state if result if off scale */
	if(result<-1||result>2){
		
#ifdef DEBUG3
		printf("check_service() end - return code from service check out of bounds!\n");
#endif

	        return STATE_UNKNOWN;
	        }

#ifdef DEBUG0
	printf("check_service() end \n");
#endif

	return result;
	}


/******************************************************************/
/******************* ROUTE/HOST CHECK FUNCTIONS *******************/
/******************************************************************/

/* check to see if we can reach the host */
int verify_route_to_host(host *hst){
	int result;

#ifdef DEBUG0
	printf("verify_route_to_host() start\n");
#endif

	/* check the host */
	result=check_host(hst);

	/* the host was unreachable */
	if(result==HOST_UNREACHABLE)
		return HOST_UNREACHABLE;

	/* the host was doen */
	else if(result==HOST_DOWN)
		return HOST_DOWN;

#ifdef DEBUG0
	printf("verify_route_to_host() end\n");
#endif

	/* we got through to the host okay */
	return HOST_UP;
        }



/* see if the remote host is alive at all */
int check_host(host *hst){
	int result=HOST_UP;
	int parent_result=HOST_UP;
	char processed_check_command[MAX_COMMANDLINE_LENGTH];
	char raw_check_command[MAX_COMMANDLINE_LENGTH];
	host *parent_host=NULL;
	command *temp_command;
	int return_result=HOST_UP;
	time_t t;

#ifdef DEBUG0
	printf("check_host() start\n");
#endif

	/* if there is no host check command, just return with no error */
	if(!strcmp(hst->host_check_command,"")){

#ifdef DEBUG3
		printf("\tNo host check command specified, so no check will be done!\n");
#endif

		return HOST_UP;
	        }

	/* find the command we use to check the host */
	temp_command=find_command(hst->host_check_command,NULL);

	/* if we couldn't find the command, return with an error */
	if(temp_command==NULL){

#ifdef DEBUG3
		printf("\tCouldn't find the host check command!\n");
#endif

		return HOST_UP;
	        }

	/* get the raw command line */
	strcpy(raw_check_command,temp_command->command_line);

	/* grab the host macros */
	grab_host_macros(hst);

	/* get the last host check time */
	time(&t);
	hst->last_check=t;

	/* process any macros in the check command */
	process_macros(raw_check_command,&processed_check_command[0]);

	/* redirect output and errors */
	strcat(processed_check_command," > /dev/null 2> /dev/null");

	/* if the host is already down or unreachable... */
	if(hst->status!=HOST_UP){

		/* make sure we return the original host problem unless we get a recovery later on... */
		return_result=hst->status;

		for(hst->current_attempt=1;hst->current_attempt<=hst->max_attempts;hst->current_attempt++){
			
#ifdef DEBUG3
			printf("\tHost Check #%d/%d\n",hst->current_attempt,hst->max_attempts);
			printf("\t\tRaw Command: %s\n",hst->host_check_command);
			printf("\t\tProcessed Command: %s\n",processed_check_command);
#endif

			/* if we're not doing agressive host checking, don't retry the host check more than once */
			if(!use_agressive_host_checking && hst->current_attempt>=2)
				break;

			/* run the host check command */
			result=my_system(processed_check_command);

			/* if we're not doing agressive host checking, let WARNING states indicate the host is up (fake the result to be STATE_OK) */
			if(!use_agressive_host_checking && result==STATE_WARNING)
				result=STATE_OK;

			/* the host recovered from a hard problem... */
			if(result==STATE_OK){

				return_result=HOST_UP;

				/* handle the hard host recovery */
				handle_host_state(hst,HOST_UP,HARD_STATE);

				/* if the host was previously unreachable, then all parent hosts must now be okay, so start checking them */
				if(hst->status==HOST_UNREACHABLE){

				        /* find the parent host */
					parent_host=find_host(hst->parent_host,NULL);

					/* check the parent host */
					if(parent_host!=NULL)
						parent_result=check_host(parent_host);
				        }

				break;
			        }

		        }

		/* reset the current attempt if we just went over... */
		if(hst->current_attempt>hst->max_attempts)
			hst->current_attempt=hst->max_attempts;
	        }


	/* else the host is supposed to be up right now... */
	else{

		for(hst->current_attempt=1;hst->current_attempt<=hst->max_attempts;hst->current_attempt++){

#ifdef DEBUG3
			printf("\tHost Check #%d/%d\n",hst->current_attempt,hst->max_attempts);
			printf("\t\tRaw Command: %s\n",hst->host_check_command);
			printf("\t\tProcessed Command: %s\n",processed_check_command);
#endif

			/* run the host check command */
			result=my_system(processed_check_command);

			/* if this is the last check and we still haven't had a recovery, check the parent and then handle the hard host state */
			if(result!=STATE_OK && (hst->current_attempt==hst->max_attempts)){

				/* if the host has no parent, then it is down */
				if(!strcmp(hst->parent_host,""))
					return_result=HOST_DOWN;

				/* else check the parent host to determine if this host is DOWN or UNREACHABLE */
				else{

				        /* find the parent host */
					parent_host=find_host(hst->parent_host,NULL);

				        /* check the parent host, assume its up if we can't find it */
					if(parent_host!=NULL)
						parent_result=check_host(parent_host);
					else
						parent_result=HOST_UP;

				        /* if the parent is down or unreachable, this host is unreachable */
					if(parent_result!=HOST_UP)
						return_result=HOST_UNREACHABLE;

				        /* else the parent host is up, so this host must be down */
					else
						return_result=HOST_DOWN;
		                        }

				/* handle the hard host state (whether it is DOWN or UNREACHABLE) */
				handle_host_state(hst,return_result,HARD_STATE);
			        }

			/* handle any soft states */
			else if(result!=STATE_OK || (result==STATE_OK && hst->current_attempt!=1))
				handle_host_state(hst,(result==STATE_OK)?HOST_UP:HOST_DOWN,SOFT_STATE);

			/* the host recovered, so break out of the check loop */
			if(result==STATE_OK)
				break;
	                }


		/* adjust the current check number if we exceeded the max count */
		if(hst->current_attempt>hst->max_attempts)
			hst->current_attempt=hst->max_attempts;
	        }


#ifdef DEBUG3
	printf("\tHost Check Result: Host '%s' is ",hst->name);
	if(return_result==HOST_UP)
		printf("UP\n");
	else if(return_result==HOST_DOWN)
		printf("DOWN\n");
	else if(return_result==HOST_UNREACHABLE)
		printf("UNREACHABLE\n");
	else
		printf("UNKNOWN! (This shouldn't be happening)\n");
#endif

#ifdef DEBUG0
	printf("check_host() end\n");
#endif
	
	return return_result;
        }



/* top level host state handler */
int handle_host_state(host *hst,int state,int state_type){
	int state_change=FALSE;
	char temp_buffer[MAX_INPUT_BUFFER];


#ifdef DEBUG0
	printf("handle_host_state() start\n");
#endif

	/* grab the host macros */
	grab_host_macros(hst);

	/* get the host state type macro */
	strcpy(macro_state_type,(state_type==HARD_STATE)?"HARD":"SOFT");

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* has the host state changed? */
	if(hst->status!=state)
		state_change=TRUE;

	/* if the host state has changed... */
	if(state_change==TRUE){

		/* the host just recovered */
		if(state==HOST_UP){

			/* reset the current host attempt number */
			hst->current_attempt=1;
		        }

		/* write the host state change to the main log file */
		if(state_type==HARD_STATE || (state_type==SOFT_STATE && log_host_retries)){
			sprintf(temp_buffer,"HOST ALERT: %s;%s;%s;%s\n",hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,macro_state_type,macro_current_host_attempt);
			write_to_log(temp_buffer);
		        }

		/* notify contacts about the recovery or problem if its a "hard" state */
		if(state_type==HARD_STATE)
			host_notification(hst,state);

		/* handle the host state change */
		handle_host_event(hst,state,state_type);

		/* set the host state flag and update the status log if this is a hard state */
		if(state_type==HARD_STATE){
			hst->status=state;
			update_status_log_with_host(hst,state);
		        }
	        }

	/* else the host state has not changed */
	else{

		/* notify contacts if host is still down */
		if(state!=HOST_UP && state_type==HARD_STATE)
			host_notification(hst,state);

		/* if we're in a soft state and we should log host retries, do so now... */
		if(state_type==SOFT_STATE && log_host_retries){

			sprintf(temp_buffer,"HOST ALERT: %s;%s;%s;%s\n",hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,macro_state_type,macro_current_host_attempt);
			write_to_log(temp_buffer);
		        }

		/* update the status log if this is a hard state */
		if(state_type==HARD_STATE)
			update_status_log_with_host(hst,state);

	        }

#ifdef DEBUG0
	printf("handle_host_state() end\n");
#endif

	return OK;
        }



/******************************************************************/
/******************** EVENT HANDLER FUNCTIONS *********************/
/******************************************************************/

int handle_service_event(service *svc,int state,int state_type){
	command *event_command;
	int state_change;

#ifdef DEBUG0
	printf("handle_service_event() start\n");
#endif


	/* has the service state changed? */
	if(svc->last_state!=state)
		state_change=TRUE;
	else
		state_change=FALSE;

	/* if service state hasn't changed, don't run the event handler */
	if(state_change==FALSE && state_type==HARD_STATE){

#ifdef DEBUG3
		printf("\tService state hasn't changed - Event handler will not be executed\n");
#endif

		return OK;
	        }

	/* if no event handler is defined, we can't do anything */
	if(!strcmp(svc->event_handler,"")){

#ifdef DEBUG3
		printf("\tNo event handler command has been defined for service '%s'\n",svc->description);
#endif

		return OK;
	        }
	        

	/* find the command */
	event_command=find_command(svc->event_handler,NULL);
	if(event_command==NULL){

#ifdef DEBUG3
		printf("\tCould not find event handler command '%s' for service '%s'!\n",svc->event_handler,svc->description);
#endif
		
		return ERROR;
	        }

	/* grab the current service check number macro */
	sprintf(macro_current_service_attempt,"%d",svc->current_attempt);

	/* run the event handler command */
	run_service_event_handler(svc,event_command,state,state_type);

#ifdef DEBUG0
	printf("handle_service_event() end\n");
#endif

	return OK;
        }


int run_service_event_handler(service *svc,command *cmd,int state,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];
	char *service_status;

#ifdef DEBUG0
	printf("run_service_event_handler() start\n");
#endif

	/* update service macros */
	grab_service_macros(svc,state);

	/* get the raw command line to execute */
	strcpy(raw_command_line,cmd->command_line);

#ifdef DEBUG3
	printf("\tRaw event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line);

#ifdef DEBUG3
	printf("\tProcessed event handler command line: %s\n",processed_command_line);
#endif

	switch(state){
	case STATE_OK:
		service_status="RECOVERY";
		break;
	case STATE_WARNING:
		service_status="WARNING";
		break;
	case STATE_CRITICAL:
		service_status="CRITICAL";
		break;
	default:
		service_status="UNKNOWN";
		break;
	        }

	if(log_event_handlers){
		sprintf(temp_buffer,"SERVICE EVENT HANDLER: %s;%s;%s;%s;%d;%s\n",svc->host_name,svc->description,service_status,(state_type==SOFT_STATE)?"SOFT":"HARD",svc->current_attempt,svc->event_handler);
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_service_event_handler() end\n");
#endif

	return OK;
        }


int handle_host_event(host *hst,int state,int state_type){
	command *event_command;
	int state_change;

#ifdef DEBUG0
	printf("handle_host_event() start\n");
#endif

	/* has the host state changed? */
	if(hst->status!=state)
		state_change=TRUE;
	else
		state_change=FALSE;

	/* if host state hasn't changed, don't run the event handler */
	if(state_change==FALSE && state_type==HARD_STATE){

#ifdef DEBUG3
		printf("\tHost state hasn't changed - Event handler will not be executed\n");
#endif

		return OK;
	        }

	/* if no event handler is defined, we can't do anything */
	if(!strcmp(hst->event_handler,"")){

#ifdef DEBUG3
		printf("\tNo event handler command has been defined for host '%s'\n",hst->name);
#endif

		return OK;
	        }
	        

	/* find the command */
	event_command=find_command(hst->event_handler,NULL);
	if(event_command==NULL){

#ifdef DEBUG3
		printf("\tCould not find event handler command '%s' for host '%s'!\n",hst->event_handler,hst->name);
#endif
		
		return ERROR;
	        }

	/* run the event handler command */
	run_host_event_handler(hst,event_command,state,state_type);

#ifdef DEBUG0
	printf("handle_host_event() end\n");
#endif

	return OK;
        }


int run_host_event_handler(host *hst,command *cmd,int state,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("run_host_event_handler() start\n");
#endif

	/* update host macros */
	grab_host_macros(hst);

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* get the raw command line to execute */
	strcpy(raw_command_line,cmd->command_line);

#ifdef DEBUG3
	printf("\tRaw event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line);

#ifdef DEBUG3
	printf("\tProcessed event handler command line: %s\n",processed_command_line);
#endif

	if(log_event_handlers){
		sprintf(temp_buffer,"HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_host_state,(state_type==SOFT_STATE)?"SOFT":"HARD",macro_current_host_attempt,hst->event_handler);
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_host_event_handler() end\n");
#endif

	return OK;
        }



/******************************************************************/
/********************* NOTIFICATION FUNCTIONS *********************/
/******************************************************************/

/* notify contacts about a service problem or recovery */
int service_notification(service *svc,int state){
	host *temp_host;
	notification *temp_notification;
	time_t current_time;

#ifdef DEBUG0
	printf("service_notification() start\n");
#endif

	/* find the host this service is associated with */
	temp_host=find_host(svc->host_name,NULL);

	/* if we couldn't find the host, return an error */
	if(temp_host==NULL)
		return ERROR;

	/* if the host is down or unreachable, don't notify contacts about multiple service failures */
	if(temp_host->status!=HOST_UP){
#ifdef DEBUG4
		printf("\tThe host is either down or unreachable, so we won't notify contacts about this service!\n");
#endif
		return OK;
	        }
	
	/* see if we should notify about problems with this service */
	if((state==STATE_WARNING||state==STATE_UNKNOWN) && !svc->notify_on_warning){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about WARNING or UNKNOWN states for this service!\n");
#endif
		return OK;
	        }
	if(state==STATE_CRITICAL && !svc->notify_on_critical){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about CRITICAL states for this service!\n");
#endif
		return OK;
	        }
	if(state==STATE_OK){
		if(!svc->notify_on_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about RECOVERY states for this service!\n");
#endif
			return OK;
		        }
		if((svc->last_state==STATE_WARNING||svc->last_state==STATE_UNKNOWN) && !svc->notify_on_warning){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from warning or unknown states for this service!\n");
#endif
			return OK;
		        }
		if(svc->last_state==STATE_CRITICAL && !svc->notify_on_critical){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from critical states for this service!\n");
#endif
			return OK;
		        }
	        }


	/* see if the service can have notifications sent out at this time */
	if(!check_time_against_period((unsigned long)time(NULL),svc->notification_period)){
#ifdef DEBUG4
		printf("\tThis service shouldn't have notifications sent out at this time!\n");
#endif
		return OK;
	        }

	/* last notification time is null when the last check was okay, so the next check
	   is always true if a service just went down (i.e. the contacts *will* be notified) */
	if(svc->last_state==STATE_OK)
		svc->last_notification=0L;

	/* don't notify if we haven't waited long enough since the last time */
	time(&current_time);
	if(svc->last_notification+(svc->notification_interval*interval_length)>current_time){
#ifdef DEBUG4
		printf("\tWe haven't waited long enough to re-notify contacts about this service!\n");
#endif
		return OK;
	        }

	/* grab the macro variables */
	grab_service_macros(svc,state);
	grab_host_macros(temp_host);

	/* create the contact notification list for this service */
	create_notification_list_from_service(svc);

	/* notify each contact (duplicates have been removed) */
	for(temp_notification=notification_list;temp_notification!=NULL;temp_notification=temp_notification->next){

		/* grab the macro variables for this contact */
		grab_contact_macros(temp_notification->contact);

		/* notify this contact */
		notify_contact_of_service(temp_notification->contact,svc,state);
	        }

	/* free memory allocated to the notification list */
	free_notification_list();

	/* update the last notification time for this service */
	time(&current_time);
	svc->last_notification=current_time;

#ifdef DEBUG0
	printf("service_notification() end\n");
#endif

	return OK;
	}


/* notify a specific contact about a service problem or recovery */
int notify_contact_of_service(contact *cntct,service *svc,int state){
	char *temp_command_name;
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char raw_commands[MAX_CONTACTCOMMANDS_LENGTH];
	char command_line[MAX_COMMANDLINE_LENGTH];
	char raw_summary[MAX_COMMANDLINE_LENGTH];
	char temp_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_COMMANDLINE_LENGTH+MAX_INPUT_BUFFER];
	char service_state[MAX_INPUT_BUFFER];
	/*char *buffer;*/
	int current_command;
	int x;

#ifdef DEBUG0
	printf("notify_contact_of_service() start\n");
#endif
#ifdef DEBUG4
	printf("\tNotify user %s\n",cntct->name);
#endif


	/* see if we should notify about problems with this service */
	if((state==STATE_WARNING||state==STATE_UNKNOWN) && !cntct->notify_on_service_warning){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about WARNING or UNKNOWN states!\n");
#endif
		return OK;
	        }

	if(state==STATE_CRITICAL && !cntct->notify_on_service_critical){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about CRITICAL states!\n");
#endif
		return OK;
	        }

	if(state==STATE_OK){

		if(!cntct->notify_on_service_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about RECOVERY states!\n");
#endif
			return OK;
		        }

		if((svc->last_state==STATE_WARNING||svc->last_state==STATE_UNKNOWN) && !cntct->notify_on_service_warning){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from warning or unknown states!\n");
#endif
			return OK;
		        }

		if(svc->last_state==STATE_CRITICAL && !cntct->notify_on_service_critical){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from critical states!\n");
#endif
			return OK;
		        }
	        }

	/* see if the contact can be notified at this time */
	if(!check_time_against_period((unsigned long)time(NULL),cntct->service_notification_period)){
#ifdef DEBUG4
		printf("\tThis contact shouldn't be notified at this time!\n");
#endif
		return OK;
	        }

	/* get all the notification commands for this contact */
	strcpy(raw_commands,cntct->commands);

	/* process all the notification commands this user has */
	current_command=0;
	while(1){

		/* increment the current command number */
		current_command++;

		/**** THIS IS A TOTAL HACK TO GET THINGS TO WORK! ****/
		strcpy(raw_commands,cntct->commands);
		temp_command_name=my_strtok(raw_commands,",");
		for(x=1;x<current_command;x++)
			temp_command_name=my_strtok(NULL,",");

		/* break out of the loop if there are no more commands */
		if(temp_command_name==NULL)
			break;

		/* get the raw command line */
		strcpy(temp_command_line,temp_command_name);
		get_raw_command_line(temp_command_line,&raw_command_line[0]);

		/* run the notification command */
		if(strcmp(raw_command_line,"")){

			/* create summary macro */
			strcpy(raw_summary,generic_summary);
			process_macros(raw_summary,&macro_summary[0]);

#ifdef DEBUG4
			printf("\tRaw Command:       %s\n",raw_command_line);
#endif

			/* replace macros in the command line */
			/*strcpy(raw_command_line,buffer);*/
			process_macros(raw_command_line,&command_line[0]);

#ifdef DEBUG4
			printf("\tProcessed Command: %s\n",command_line);
#endif

			if(state==STATE_OK)
				strcpy(service_state,"RECOVERY");
			else if(state==STATE_CRITICAL)
				strcpy(service_state,"CRITICAL");
			else if(state==STATE_WARNING)
				strcpy(service_state,"WARNING");
			else
				strcpy(service_state,"UNKNOWN");

			/* log the notification to program log file */
			if(log_notifications){
				sprintf(temp_buffer,"SERVICE NOTIFICATION: %s;%s;%s;%s;%s;%s\n",cntct->name,svc->host_name,svc->description,service_state,temp_command_line,macro_output);
				write_to_log(temp_buffer);
			        }

			/* run the command */
			my_system(command_line);
		        }

		/* this shouldn't happen, but just in case... */
		else{

#ifdef DEBUG1
			printf("\tHey, command #%d was NULL!\n",current_command);
#endif
		        }
	        }

#ifdef DEBUG0
	printf("notify_contact_of_service() end\n");
#endif

	return OK;
        }



/* notify all contacts for a host that the entire host is down or up */
int host_notification(host *hst,int state){
	notification *temp_notification;
	time_t current_time;
	int state_change;


#ifdef DEBUG0
	printf("host_notification() start\n");
#endif

	/* get the current time */
	time(&current_time);

	/* has the host state changed? */
	if(hst->status!=state)
		state_change=TRUE;
	else
		state_change=FALSE;


	/* see if we should notify about problems with this host */
	if((state==HOST_UNREACHABLE) && !hst->notify_on_unreachable){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about UNREACHABLE status for this host!\n");
#endif
		return OK;
	        }
	if(state==HOST_DOWN && !hst->notify_on_down){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about DOWN states for this host!\n");
#endif
		return OK;
	        }
	if(state==HOST_UP){

		if(!hst->notify_on_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about RECOVERY states for this host!\n");
#endif
			return OK;
		       }

		if(hst->status==HOST_UNREACHABLE && !hst->notify_on_unreachable){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from unreachable states for this host!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_DOWN && !hst->notify_on_down){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from down states for this host!\n");
#endif
			return OK;
		        }
	        }


	/* see if the host can have notifications sent out at this time */
	if(!check_time_against_period((unsigned long)time(NULL),hst->notification_period)){
#ifdef DEBUG4
		printf("\tThis host shouldn't have notifications sent out at this time!\n");
#endif
		return OK;
	        }

	/* if we haven't notified the contacts about the host state change, do so */
	if(state_change==TRUE){

#ifdef DEBUG3
		printf("\tHOST STATE CHANGE!\n");
#endif
	        }

	/* else if its time to re-notify the contacts about the host... */
	else if(state==HOST_DOWN && current_time>hst->last_host_notification+(hst->notification_interval*interval_length)){

#ifdef DEBUG3
		printf("\tTime to re-notify the contacts about host being down!\n");
#endif
	        }

	/* else we shouldn't do the notification, so exit */
	else{

#ifdef DEBUG3
		printf("\tIts not yet time to re-notify the contacts about this host being down...\n");
#endif

		return OK;
	        }

	/* grab the macro variables */
	grab_host_macros(hst);

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* create the contact notification list for this host */
	create_notification_list_from_host(hst);

	/* notify each contact (duplicates have been removed) */
	for(temp_notification=notification_list;temp_notification!=NULL;temp_notification=temp_notification->next){

		/* grab the macro variables for this contact */
		grab_contact_macros(temp_notification->contact);

		/* notify this contact */
		notify_contact_of_host(temp_notification->contact,hst,state);
	        }

	/* free memory allocated to the notification list */
	free_notification_list();

	/* update the last notification time for this host */
	time(&current_time);
	hst->last_host_notification=current_time;

#ifdef DEBUG0
	printf("host_notification() end\n");
#endif

	return OK;
        }


/* notify a specific contact that an entire host is down or up */
int notify_contact_of_host(contact *cntct,host *hst,int state){
	char temp_buffer[MAX_INPUT_BUFFER];
	char *temp_command_name;
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char raw_commands[MAX_CONTACTCOMMANDS_LENGTH];
	char command_line[MAX_COMMANDLINE_LENGTH];
	char raw_summary[MAX_COMMANDLINE_LENGTH];
	char temp_command_line[MAX_COMMANDLINE_LENGTH];
	/*char *buffer;*/
	int current_command;
	int x;

#ifdef DEBUG0
	printf("notify_contact_of_host() start\n");
#endif
#ifdef DEBUG4
	printf("\tNotify user %s\n",cntct->name);
#endif

	/* see if we should notify about problems with this host */
	if((state==HOST_DOWN) && !cntct->notify_on_host_down){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about DOWN states!\n");
#endif
		return OK;
	        }

	if(state==HOST_UNREACHABLE && !cntct->notify_on_host_unreachable){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about UNREACHABLE states!\n");
#endif
		return OK;
	        }

	if(state==HOST_UP){

		if(!cntct->notify_on_host_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about RECOVERY states!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_DOWN && !cntct->notify_on_host_down){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from down states!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_UNREACHABLE && !cntct->notify_on_host_unreachable){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from unreachable states!\n");
#endif
			return OK;
		        }
	        }


	/* see if the contact can be notified at this time */
	if(!check_time_against_period((unsigned long)time(NULL),cntct->host_notification_period)){
#ifdef DEBUG4
		printf("\tThis contact shouldn't be notified at this time!\n");
#endif
		return OK;
	        }

	/* get all the notification commands for this contact */
	strcpy(raw_commands,cntct->host_commands);

	/* process all the notification commands this user has */
	current_command=0;
	while(1){

		/* increment the current command number */
		current_command++;

		/**** THIS IS A TOTAL HACK TO GET THINGS TO WORK! ****/
		strcpy(raw_commands,cntct->host_commands);
		temp_command_name=my_strtok(raw_commands,",");
		for(x=1;x<current_command;x++)
			temp_command_name=my_strtok(NULL,",");

		/* break out of the loop if there are no more commands */
		if(temp_command_name==NULL)
			break;

		/* get the raw command line */
		strcpy(temp_command_line,temp_command_name);
		get_raw_command_line(temp_command_line,&raw_command_line[0]);

		/* run the notification command */
		if(strcmp(raw_command_line,"")){

			/* create summary macro */
			strcpy(raw_summary,generic_summary);
			process_macros(raw_summary,&macro_summary[0]);

#ifdef DEBUG4
			printf("\tRaw Command:       %s\n",raw_command_line);
#endif
			/* replace macros in the command line */
			/*strcpy(raw_command_line,buffer);*/
			process_macros(raw_command_line,&command_line[0]);

#ifdef DEBUG4
			printf("\tProcessed Command: %s\n",command_line);
#endif

			/* log the notification to program log file */
			if(log_notifications){
				sprintf(temp_buffer,"HOST NOTIFICATION: %s;%s;%s;%s;Host is %s\n",cntct->name,hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,temp_command_line,macro_host_state);
				write_to_log(temp_buffer);
			        }

			/* run the command */
			my_system(command_line);
		        }

		/* this shouldn't happen, but just in case... */
		else{

#ifdef DEBUG1
			printf("\tHey, command #%d was NULL!\n",current_command);
#endif
		        }
	        }


#ifdef DEBUG0
	printf("notify_contact_of_host() end\n");
#endif

	return OK;
        }


/* given a host, create a list of contacts to be notified, removing duplicates */
int create_notification_list_from_host(host *hst){
	hostgroup *temp_hostgroup;
	hostgroupmember *temp_hostgroupmember;
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	contact *temp_contact;
	char contact_groups[MAX_CONTACTGROUPS_LENGTH];
	char *temp_groupname;

	/* check all the hostgroups for this host */
	temp_hostgroup=hostgroup_list;
	while(temp_hostgroup!=NULL){

		/* see if the host if a member of this hostgroup */
		temp_hostgroupmember=NULL;
		temp_hostgroupmember=find_hostgroupmember(hst->name,temp_hostgroup,temp_hostgroupmember);

		/* we found a hostgroup that includes this host */
		if(temp_hostgroupmember!=NULL){

			/* find all contact groups for this hostgroup */
			strcpy(contact_groups,temp_hostgroup->contact_groups);
			for(temp_groupname=my_strtok(contact_groups,",");temp_groupname!=NULL;temp_groupname=my_strtok(NULL,",")){

				/* find the contact group */
				temp_contactgroup=find_contactgroup(temp_groupname,NULL);

				/* if we couldn't find the contact group, skip it */
				if(temp_contactgroup==NULL)
					continue;

				/* all all contacts in this contact group to the notification list (ignoring duplicates) */
				temp_contactgroupmember=temp_contactgroup->members;
				while(temp_contactgroupmember!=NULL){

					/* add the contact to the notification list */
					temp_contact=NULL;
					temp_contact=find_contact(temp_contactgroupmember->contact_name,temp_contact);
					if(temp_contact!=NULL)
						add_notification(temp_contact);

					temp_contactgroupmember=temp_contactgroupmember->next;
				        }
			        }
	                }

		/* find the next hostgroup */
		temp_hostgroup=temp_hostgroup->next;
	        }

	return OK;
        }



/* given a service, create a list of contacts to be notified, removing duplicates */
int create_notification_list_from_service(service *svc){
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	contact *temp_contact;
	char contact_groups[MAX_CONTACTGROUPS_LENGTH];
	char *temp_groupname;


	/* find all contact groups for this service */
	strcpy(contact_groups,svc->contact_groups);
	for(temp_groupname=my_strtok(contact_groups,",");temp_groupname!=NULL;temp_groupname=my_strtok(NULL,",")){

		/* find the contact group */
		temp_contactgroup=find_contactgroup(temp_groupname,NULL);

		/* if we couldn't find the contact group, skip it */
		if(temp_contactgroup==NULL)
			continue;

		/* all all contacts in this contact group to the notification list (ignoring duplicates) */
		temp_contactgroupmember=temp_contactgroup->members;
		while(temp_contactgroupmember!=NULL){

			/* add the contact to the notification list */
			temp_contact=NULL;
			temp_contact=find_contact(temp_contactgroupmember->contact_name,temp_contact);
			if(temp_contact!=NULL)
				add_notification(temp_contact);

			temp_contactgroupmember=temp_contactgroupmember->next;
		        }
	        }

	return OK;
        }



/******************************************************************/
/************************ LOGGING FUNCTIONS ***********************/
/******************************************************************/

/* write something to the netsaint log file */
int write_to_log(char *buffer){
	FILE *fp;
	time_t t;

#ifdef DEBUG0
	printf("write_to_log() start\n");
#endif

	/* don't log anything if we are just verifying the configuration */
	if(verify_config==TRUE)
		return OK;

	/* make sure log file has been specified */
	if(!strcmp(log_file,"")){
		printf("Warning: Log file has not been specified - cannot write to log!\n");
		return ERROR;
	        }

	fp=fopen(log_file,"a+");
	if(fp==NULL){
		printf("Warning: Cannot open log file '%s' for writing\n",log_file);
		return ERROR;
		}

	/* get the current time */
	time(&t);

	/* strip any newlines from the end of the buffer */
	strip(buffer);

	/* write the buffer to the log file */
	fprintf(fp,"[%lu] %s\n",t,buffer);

	fclose(fp);

#ifdef DEBUG0
	printf("write_to_log() end\n");
#endif

	return OK;
	}


/* write something to the syslog facility */
int write_to_syslog(char *buffer){

#ifdef DEBUG0
	printf("write_to_syslog() start\n");
#endif

	/* bail out if we shouldn't write to syslog */
	if(!use_syslog)
		return OK;

	/* open the syslog */
	syslog(LOG_USER|LOG_INFO,"%s",buffer);

#ifdef DEBUG0
	printf("write_to_syslog() end\n");
#endif

	return OK;
	}


/* write a service problem/recovery to the netsaint log file */
int write_event_to_log(service *svc,int state){
	char buffer[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char temp_state[32];

#ifdef DEBUG0
	printf("write_event_to_log() start\n");
#endif

	/* don't log warnings or unknown errors if we only report critical stuff */
	if((state==STATE_WARNING||state==STATE_UNKNOWN)&&log_level==2)
		return OK;

	if(state==STATE_UNKNOWN)
		strcpy(temp_state,"UNKNOWN");
	else if(state==STATE_OK)
		strcpy(temp_state,"RECOVERY");
	else if(state==STATE_WARNING)
		strcpy(temp_state,"WARNING");
	else if(state==STATE_CRITICAL)
		strcpy(temp_state,"CRITICAL");

	/* don't log soft errors if the user doesn't want to */
	if(svc->state_type==SOFT_STATE && !log_service_retries)
		return OK;

	/* don't log if we already logged this device being critical or at warning level */
	if(svc->last_state==state && svc->error_was_logged==TRUE)
		return OK;

	sprintf(temp_buffer,"SERVICE ALERT: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,temp_state,(svc->state_type==SOFT_STATE)?"SOFT":"HARD",macro_current_service_attempt,macro_output);
	strcpy(buffer,temp_buffer);

	write_to_log(buffer);

#ifdef DEBUG0
	printf("write_event_to_log() end\n");
#endif

	return OK;
	}


/* write a service problem/recovery to the syslog facility */
int write_event_to_syslog(service *svc,int state){
	char buffer[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char temp_state[32];

#ifdef DEBUG0
	printf("write_event_to_syslog() start\n");
#endif

	/* bail out if we shouldn't write to syslog */
	if(!use_syslog)
		return OK;

	/* don't log warnings or unknown errors if we only report critical stuff */
	if((state==STATE_WARNING||state==STATE_UNKNOWN)&&log_level==2)
		return OK;

	/* don't log service retries if the user doesn't want to */
	if(svc->state_type==SOFT_STATE && !log_service_retries)
		return OK;

	/* don't log if we already logged this device being critical or at warning level */
	if(svc->last_state==state && svc->error_was_logged==TRUE)
		return OK;

	if(state==STATE_UNKNOWN)
		strcpy(temp_state,"UNKNOWN");
	else if(state==STATE_OK)
		strcpy(temp_state,"RECOVERY");
	else if(state==STATE_WARNING)
		strcpy(temp_state,"WARNING");
	else if(state==STATE_CRITICAL)
		strcpy(temp_state,"CRITICAL");

	sprintf(temp_buffer,"SERVICE ALERT: %s;%s;%s;%s;%s;\n",svc->host_name,svc->description,temp_state,(svc->state_type==SOFT_STATE)?"SOFT":"HARD",macro_output);
	strcpy(buffer,temp_buffer);

	write_to_syslog(buffer);

#ifdef DEBUG0
	printf("write_event_to_syslog() end\n");
#endif

	return OK;
	}


/* update the status log with the current status of a particular services */
int update_status_log_with_service(service *svc,int state){
	FILE *fp,*fp2;
	char input_buffer[MAX_INPUT_BUFFER];
	char state_string[MAX_INPUT_BUFFER];
	char service_string[MAX_INPUT_BUFFER];
	char host_string[MAX_INPUT_BUFFER];
	char new_service_string[MAX_INPUT_BUFFER];
	char new_host_string[MAX_INPUT_BUFFER];
	host *temp_host;
	int rename_result=0;

#ifdef DEBUG0
	printf("update_status_log_with_service() start\n");
#endif

	/* get the host associated with this service */
	temp_host=NULL;
	temp_host=find_host(svc->host_name,temp_host);

	/* get the service state string */
	if(temp_host->status==HOST_DOWN)
		strcpy(state_string,"HOST DOWN");
	else if(temp_host->status==HOST_UNREACHABLE)
		strcpy(state_string,"UNREACHABLE");
	else if(state==STATE_OK&&svc->last_state==STATE_OK)
		strcpy(state_string,"OK");
	else if(state==STATE_OK)
		strcpy(state_string,"RECOVERY");
	else if(state==STATE_CRITICAL)
		strcpy(state_string,"CRITICAL");
	else if(state==STATE_WARNING)
		strcpy(state_string,"WARNING");
	else
		strcpy(state_string,"UNKNOWN");


	/* create the service status string */
	sprintf(new_service_string,"[%lu] SERVICE;%s;%s;%s;%d/%d;%lu;%s\n",svc->last_check,svc->host_name,svc->description,state_string,(svc->state_type==SOFT_STATE)?svc->current_attempt-1:svc->current_attempt,svc->max_attempts,svc->next_check,macro_output);

	/* create a service string to identify previous entries in the log for this service/host pair */
	sprintf(service_string,"] SERVICE;%s;%s;",svc->host_name,svc->description);

	/* get the host state string */
	if(temp_host->status==HOST_UP){

		strcpy(state_string,"UP");

		/* create the host status string */
		sprintf(new_host_string,"[%lu] HOST;%s;%s;Host appears to be UP (Service '%s' was just checked)\n",svc->last_check,temp_host->name,state_string,svc->description);
	        }
	else{

		if(temp_host->status==HOST_DOWN)
			strcpy(state_string,"DOWN");
		else if(temp_host->status==HOST_UNREACHABLE)
			strcpy(state_string,"UNREACHABLE");

		/* create the host status string */
		sprintf(new_host_string,"[%lu] HOST;%s;%s;Host is %s\n",svc->last_check,temp_host->name,state_string,state_string);
	        }

	/* create a host string to identify previous entries in the log file for this host */
	sprintf(host_string,"] HOST;%s;",temp_host->name);


	/* create a temp file for writing the new status log */
	fp2=fopen(temp_file,"w");
	if(fp2==NULL){
		printf("Error: Could not create temp file '%s'!\n",temp_file);
		return ERROR;
	        }

	/* open the existing status log */
	fp=fopen(status_file,"r");
	if(fp!=NULL){

		/* read all lines from the original status log */
		while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

			/* if this is the old host string, write the new one instead */
			if(strstr(input_buffer,host_string))
				fputs(new_host_string,fp2);

			/* if this is the old service string, write the new one instead */
			else if(strstr(input_buffer,service_string))
				fputs(new_service_string,fp2);

			/* if this is not a duplicate entry, write it to the new log file */
			else
				fputs(input_buffer,fp2);
		        }

		fclose(fp);
	        }

	fclose(fp2);

	/* move the temp file to the status log (overwrite the old status log) */
	rename_result=rename(temp_file,status_file);

	if(rename_result){
		printf("Error: Could not rename() %s to %s\n",temp_file,status_file);
		return ERROR;
	        }

#ifdef DEBUG0
	printf("update_status_log_with_service() end\n");
#endif

	return OK;
        }




/* update the status log with the current status of a particular host */
int update_status_log_with_host(host *hst,int state){
	FILE *fp,*fp2;
	char input_buffer[MAX_INPUT_BUFFER];
	char state_string[MAX_INPUT_BUFFER];
	char host_string[MAX_INPUT_BUFFER];
	char new_host_string[MAX_INPUT_BUFFER];
	int rename_result=0;

#ifdef DEBUG0
	printf("update_status_log_with_host() start\n");
#endif

	/* get the host state string */
	if(hst->status==HOST_DOWN)
		strcpy(state_string,"DOWN");
	else if(hst->status==HOST_UNREACHABLE)
		strcpy(state_string,"UNREACHABLE");
	else
		strcpy(state_string,"UP");

	/* create the host status string */
	sprintf(new_host_string,"[%lu] HOST;%s;%s;Host is %s\n",hst->last_check,hst->name,state_string,state_string);

	/* create a host string to identify previous entries in the log file for this host */
	sprintf(host_string,"] HOST;%s;",hst->name);

	/* create a temp file for writing the new status log */
	fp2=fopen(temp_file,"w");
	if(fp2==NULL){
		printf("Error: Could not create temp file '%s'!\n",temp_file);
		return ERROR;
	        }

	/* open the existing status log */
	fp=fopen(status_file,"r");
	if(fp!=NULL){

		/* read all lines from the original status log */
		while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

			/* if this is the old host string, write the new one instead */
			if(strstr(input_buffer,host_string))
				fputs(new_host_string,fp2);

			/* if this is not a duplicate entry, write it to the new log file */
			else
				fputs(input_buffer,fp2);
		        }

		fclose(fp);
	        }

	fclose(fp2);

	/* move the temp file to the status log (overwrite the old status log) */
	rename_result=rename(temp_file,status_file);

	if(rename_result){
		printf("Error: Could not rename() %s to %s\n",temp_file,status_file);
		return ERROR;
	        }

#ifdef DEBUG0
	printf("update_status_log_with_host() end\n");
#endif

	return OK;
        }



int create_initial_status_log(void){
	service *temp_service;
	host *temp_host;
	char temp_buffer[MAX_INPUT_BUFFER];
	time_t next_check;
	FILE *fp;

#ifdef DEBUG0
	printf("create_initial_status_log() start\n");
#endif

	/* open the status log for writing */
	fp=fopen(status_file,"w");
	if(fp==NULL){

#ifdef DEBUG1
		printf("Error: Could not open status log for writing\n");
#endif

		return ERROR;
	        }

	/* loop through all the hosts */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		sprintf(temp_buffer,"[N/A] HOST;%s;PENDING;Service check pending\n",temp_host->name);
		fputs(temp_buffer,fp);
	        }

	/* loop through all the services */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		next_check=(time_t)temp_service->next_check;
		sprintf(temp_buffer,"[N/A] SERVICE;%s;%s;PENDING;0/%d;N/A;Service check scheduled for %s",temp_service->host_name,temp_service->description,temp_service->max_attempts,ctime(&next_check)); 
		fputs(temp_buffer,fp);
                }

	/* close the status log */
	fclose(fp);

#ifdef DEBUG0
	printf("create_initial_status_log() end\n");
#endif

	return OK;
        }


/******************************************************************/
/************************ MACRO FUNCTIONS *************************/
/******************************************************************/

/* replace macros in notification commands with their values */
int process_macros(char *input_buffer,char *output_buffer){
	char *temp_buffer;
	int in_macro;

#ifdef DEBUG0
	printf("process_macros() start\n");
#endif

	strcpy(output_buffer,"");

	in_macro=FALSE;
	temp_buffer=my_strtok(input_buffer,"$");
	while(temp_buffer!=NULL){

		if(in_macro==FALSE){
			if(strlen(output_buffer)+strlen(temp_buffer)<MAX_COMMANDLINE_LENGTH-1)
				strcat(output_buffer,temp_buffer);
			in_macro=TRUE;
			}
		else{

			if(strlen(output_buffer)+strlen(temp_buffer)<MAX_COMMANDLINE_LENGTH-1){

				if(!strcmp(temp_buffer,"HOSTNAME"))
					strcat(output_buffer,macro_host_name);

				else if(!strcmp(temp_buffer,"HOSTALIAS"))
					strcat(output_buffer,macro_host_alias);

				else if(!strcmp(temp_buffer,"HOSTADDRESS"))
					strcat(output_buffer,macro_host_address);

				else if(!strcmp(temp_buffer,"SERVICEDESC"))
					strcat(output_buffer,macro_service_description);

				else if(!strcmp(temp_buffer,"SERVICESTATE"))
					strcat(output_buffer,macro_service_state);

				else if(!strcmp(temp_buffer,"SERVICEATTEMPT"))
					strcat(output_buffer,macro_current_service_attempt);

				else if(!strcmp(temp_buffer,"DATETIME"))
					strcat(output_buffer,macro_date_time);

				else if(!strcmp(temp_buffer,"OUTPUT"))
					strcat(output_buffer,macro_output);

				else if(!strcmp(temp_buffer,"CONTACTNAME"))
					strcat(output_buffer,macro_contact_name);

				else if(!strcmp(temp_buffer,"CONTACTALIAS"))
					strcat(output_buffer,macro_contact_alias);

				else if(!strcmp(temp_buffer,"CONTACTEMAIL"))
					strcat(output_buffer,macro_contact_email);

				else if(!strcmp(temp_buffer,"CONTACTPAGER"))
					strcat(output_buffer,macro_contact_pager);

				else if(!strcmp(temp_buffer,"ADMINEMAIL"))
					strcat(output_buffer,macro_admin_email);

				else if(!strcmp(temp_buffer,"ADMINPAGER"))
					strcat(output_buffer,macro_admin_pager);

				else if(!strcmp(temp_buffer,"SUMMARY"))
					strcat(output_buffer,macro_summary);

				else if(!strcmp(temp_buffer,"HOSTSTATE"))
					strcat(output_buffer,macro_host_state);

				else if(!strcmp(temp_buffer,"HOSTATTEMPT"))
					strcat(output_buffer,macro_current_host_attempt);

				else if(!strcmp(temp_buffer,"STATETYPE"))
					strcat(output_buffer,macro_state_type);

				else if(!strcmp(temp_buffer,"ARG10"))
					strcat(output_buffer,macro_argv[9]);
				
				else if(!strcmp(temp_buffer,"ARG11"))
					strcat(output_buffer,macro_argv[10]);
				
				else if(!strcmp(temp_buffer,"ARG12"))
					strcat(output_buffer,macro_argv[11]);
				
				else if(!strcmp(temp_buffer,"ARG13"))
					strcat(output_buffer,macro_argv[12]);
				
				else if(!strcmp(temp_buffer,"ARG14"))
					strcat(output_buffer,macro_argv[13]);
				
				else if(!strcmp(temp_buffer,"ARG15"))
					strcat(output_buffer,macro_argv[14]);
				
				else if(!strcmp(temp_buffer,"ARG16"))
					strcat(output_buffer,macro_argv[15]);
	
				else if(!strcmp(temp_buffer,"ARG1"))
					strcat(output_buffer,macro_argv[0]);
				
				else if(!strcmp(temp_buffer,"ARG2"))
					strcat(output_buffer,macro_argv[1]);
		
				else if(!strcmp(temp_buffer,"ARG3"))
					strcat(output_buffer,macro_argv[2]);

				else if(!strcmp(temp_buffer,"ARG4"))
					strcat(output_buffer,macro_argv[3]);
				
				else if(!strcmp(temp_buffer,"ARG5"))
					strcat(output_buffer,macro_argv[4]);
				
				else if(!strcmp(temp_buffer,"ARG6"))
					strcat(output_buffer,macro_argv[5]);
				
				else if(!strcmp(temp_buffer,"ARG7"))
					strcat(output_buffer,macro_argv[6]);
				
				else if(!strcmp(temp_buffer,"ARG8"))
					strcat(output_buffer,macro_argv[7]);
				
				else if(!strcmp(temp_buffer,"ARG9"))
					strcat(output_buffer,macro_argv[8]);
			
				else{
					strcat(output_buffer,"$");
					strcat(output_buffer,temp_buffer);
					strcat(output_buffer,"$");
				        }
				}

			in_macro=FALSE;
			}

		temp_buffer=my_strtok(NULL,"$");
		}

#ifdef DEBUG0
	printf("process_macros() end\n");
#endif

	return OK;
	}


/* grab macros that are specific to a particular service */
int grab_service_macros(service *svc,int state){
	time_t t;
	
#ifdef DEBUG0
	printf("grab_service_macros() start\n");
#endif

	/* get the service description */
	strcpy(macro_service_description,svc->description);

	/* get the service state */
	if(state==STATE_OK)
		strcpy(macro_service_state,"RECOVERY");
	else if(state==STATE_WARNING)
		strcpy(macro_service_state,"WARNING");
	else if(state==STATE_CRITICAL)
		strcpy(macro_service_state,"CRITICAL");
	else if(state==STATE_UNKNOWN)
		strcpy(macro_service_state,"UNKNOWN");

	/* get the current date/time */
	time(&t);
	sprintf(macro_date_time,"%s",ctime(&t));

	strip(macro_date_time);
	strip(macro_service_description);
	strip(macro_service_state);
	sprintf(macro_current_service_attempt,"%d",svc->current_attempt);

#ifdef DEBUG0
	printf("grab_service_macros() end\n");
#endif

	return OK;
        }


/* grab macros that are specific to a particular host */
int grab_host_macros(host *hst){
	time_t t;

#ifdef DEBUG0
	printf("grab_host_macros() start\n");
#endif

	/* get the host name */
	strcpy(macro_host_name,hst->name);
	
	/* get the host alias */
	strcpy(macro_host_alias,hst->alias);

	/* get the host address */
	strcpy(macro_host_address,hst->address);

	/* get the host state */
	if(hst->status==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(hst->status==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* get the host current attempt */
	sprintf(macro_current_host_attempt,"%d",hst->current_attempt);

	strip(macro_host_state);
	strip(macro_current_host_attempt);
	strip(macro_host_name);
	strip(macro_host_alias);
	strip(macro_host_address);


	/* get the current date/time */
	time(&t);
	sprintf(macro_date_time,"%s",ctime(&t));

#ifdef DEBUG0
	printf("grab_host_macros() end\n");
#endif

	return OK;
        }


/* grab macros that are specific to a particular contact */
int grab_contact_macros(contact *cntct){
	time_t t;

#ifdef DEBUG0
	printf("grab_contact_macros() start\n");
#endif

	/* get the name */
	strcpy(macro_contact_name,cntct->name);

	/* get the alias */
	strcpy(macro_contact_alias,cntct->alias);

	/* get the email address */
	strcpy(macro_contact_email,cntct->email);

	/* get the pager number */
	strcpy(macro_contact_pager,cntct->pager);

	strip(macro_contact_name);
	strip(macro_contact_alias);
	strip(macro_contact_email);
	strip(macro_contact_pager);

	/* get the current date/time */
	time(&t);
	sprintf(macro_date_time,"%s",ctime(&t));

#ifdef DEBUG0
	printf("grab_contact_macros() end\n");
#endif

	return OK;
        }


/* clear all macros that are not "constant" (i.e. they change throughout the course of monitoring) */
int clear_volatile_macros(void){
	int x;

#ifdef DEBUG0
	printf("clear_volatile_macros() start\n");
#endif
	
	strcpy(macro_contact_name,"");
	strcpy(macro_contact_alias,"");
	strcpy(macro_contact_email,"");
	strcpy(macro_contact_pager,"");

	strcpy(macro_host_name,"");
	strcpy(macro_host_alias,"");
	strcpy(macro_host_address,"");

	strcpy(macro_service_description,"");
	strcpy(macro_service_state,"");
	strcpy(macro_current_service_attempt,"");
	strcpy(macro_current_host_attempt,"");
	strcpy(macro_host_state,"");
	strcpy(macro_state_type,"");

	strcpy(macro_date_time,"");

	for(x=0;x<MAX_COMMAND_ARGUMENTS;x++)
		strcpy(macro_argv[x],"");

#ifdef DEBUG0
	printf("clear_volatile_macros() end\n");
#endif

	return OK;
        }


/* clear macros that are constant (i.e. they do NOT change during monitoring) */
int clear_nonvolatile_macros(void){

#ifdef DEBUG0
	printf("clear_nonvolatile_macros() start\n");
#endif

	strcpy(macro_admin_email,"");
	strcpy(macro_admin_pager,"");
	strcpy(macro_summary,"");

#ifdef DEBUG0
	printf("clear_nonvolatile_macros() end\n");
#endif

	return OK;
        }



/******************************************************************/
/************************* MISC FUNCTIONS *************************/
/******************************************************************/

/* strip newline characters from end of a string */
void strip(char *buffer){
	int lastchar;

	lastchar=strlen(buffer)-1;

	while((buffer[lastchar]=='\n'||buffer[lastchar]=='\r'||buffer[lastchar]==' '||buffer[lastchar]=='\t') && lastchar>=0){
		buffer[lastchar]='\x0';
		lastchar=strlen(buffer)-1;
	        }

	return;
	}


/* fix the problem with strtok() skipping empty options between tokens */	
char *my_strtok(char *buffer,char *tokens){
	char *token_position;
	char *sequence_head;

	if(buffer!=NULL){
		if(original_my_strtok_buffer!=NULL)
			free(original_my_strtok_buffer);
		my_strtok_buffer=malloc(strlen(buffer)+1);
		if(my_strtok_buffer==NULL)
			return NULL;
		original_my_strtok_buffer=my_strtok_buffer;
		strcpy(my_strtok_buffer,buffer);
	        }
	
	sequence_head=my_strtok_buffer;

	if(sequence_head[0]=='\x0')
		return NULL;
	
	token_position=index(my_strtok_buffer,tokens[0]);

	if(token_position==NULL){
		my_strtok_buffer=index(my_strtok_buffer,'\x0');
		return sequence_head;
	        }

	token_position[0]='\x0';
	my_strtok_buffer=token_position+1;

	return sequence_head;

        }



/* fixes compiler problems under Solaris, since strsep() isn't included */
/* this code is taken from the glibc source */
char *my_strsep (char **stringp, const char *delim){
	char *begin, *end;

	begin = *stringp;
	if (begin == NULL)
		return NULL;

	/* A frequent case is when the delimiter string contains only one
	   character.  Here we don't need to call the expensive `strpbrk'
	   function and instead work using `strchr'.  */
	if(delim[0]=='\0' || delim[1]=='\0'){
		char ch = delim[0];

		if(ch=='\0')
			end=NULL;
		else{
			if(*begin==ch)
				end=begin;
			else
				end=strchr(begin+1,ch);
			}
		}

	else
		/* Find the end of the token.  */
		end = strpbrk (begin, delim);

	if(end){

		/* Terminate the token and set *STRINGP past NUL character.  */
		*end++='\0';
		*stringp=end;
		}
	else
		/* No more delimiters; this is the last token.  */
		*stringp=NULL;

	return begin;
	}



/* fixes a compiler warning under HP-UX */
char *my_index(char *buffer, char token){
	int index;

	if(buffer==NULL)
		return NULL;

	for(index=0;buffer[index]!='\x0'&&buffer[index]!=token;index++);

	return buffer+index;
        }


/* executes a system command - used for service checks and notifications */
int my_system(char *command){
        int pid;
	int status;
	int wait_result;
	int result;
	extern int errno;
	extern char **environ;
	char buffer[MAX_COMMANDLINE_LENGTH];
	time_t start_time;
	time_t current_time;
	char *argv[4];

#ifdef DEBUG0
	printf("my_system() start\n");
#endif

	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1){

		sprintf(buffer,"Warning: fork() in my_system() failed for command \"%s\"\n",command);
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif

	        return STATE_UNKNOWN;  
	        }

	/* execute the command in the child process */
        if (pid==0){

		/* Added 08/16/1999 */
		free_memory();

		argv[0]="/bin/sh";                /* Changed from 'argv[0]="sh";' on 4/16/1999 */
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;

		/* execute the command, replacing the child process */
		execve("/bin/sh",argv,environ);

		sprintf(buffer,"Warning: execve() in my_system() failed for command \"%s\"\n",command);
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif

		/* this code should not be reached */
		/* changed exit() to _exit() - 08/08/1999 */
		_exit(STATE_UNKNOWN);
	        }

	/* get the time the child process was started */
	time(&start_time);

	/* wait for the command to finish */
	while(1){

		/* see if the child process has exited, but don't wait for it to exit */
		wait_result=waitpid(pid,&status,WNOHANG | WUNTRACED);

		/* the child process finished executing */
		if(wait_result==-1)
			break;


		/* the child process has not exited yet - it is still running... */
		else if(wait_result==0){

			/* get the current time */
			time(&current_time);

			/* make sure we haven't waited too long for the child process to finish running... */
			if(current_time-start_time>max_child_process_time){

				sprintf(buffer,"Warning: A timeout occurred while trying to execute the command \"%s\"\n",command);
				write_to_log(buffer);
#ifdef DEBUG1
				printf("\t%s",buffer);
#endif

				/* attempt to kill the child process */
				kill(pid,SIGKILL);
				kill(pid,SIGTERM);

				/* attempt to clear the zombie status of the child process */
				wait_result=waitpid(pid,&status,WNOHANG | WUNTRACED);

				/* we couldn't kill the child process for some reason... */
				if(wait_result!=pid){
					sprintf(buffer,"Warning: Could not kill child process with ID %d...\n",pid);
					write_to_log(buffer);
#ifdef DEBUG1
					printf("\t%s",buffer);
#endif
				        /* increment the number of child processes that are in zombie states */
					total_orphaned_children++;
				        }

				/* set the early kill flag */
				child_process_timed_out=TRUE;

				return STATE_UNKNOWN;
			        }
			
			/* sleep for a second so we don't hog cpu time */
			sleep(1);
		        }

	        }

	/* get the exit code returned from the program */
	result=WEXITSTATUS(status);

#ifdef DEBUG1
	printf("my_system() end with status = %d,result=%d\n",status,result);
#endif

	/* if the command couldn't be found, write an error to the log file */
	if(result==126||result==127){

		sprintf(buffer,"Warning: Command \"%s\" not found\n",command);
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif
	        }

	/* check the bounds for the return code */
	if(result<0||result>2)
		return STATE_UNKNOWN;

	return result;
        }


/* given a "raw" command, return the "expanded" or "whole" command line */
void get_raw_command_line(char *cmd,char *raw_command){
	char temp_buffer[MAX_COMMANDLINE_LENGTH];
	char *buffer;
	command *temp_command;
	int x;

#ifdef DEBUG0
	printf("get_raw_command_line() start\n");
#endif
#ifdef DEBUG1
	printf("\tInput: %s\n",cmd);
#endif

	/* initialize the command */
	strcpy(raw_command,"");

	/* if we were handed a NULL string, throw it right back */
	if(cmd==NULL){
		
#ifdef DEBUG1
		printf("\tRaw command is NULL!\n");
#endif

		return;
	        }

	/* if the command is enclosed in quotes, this *is* the command, don't do a lookup */
	if(cmd[0]=='\"'){
		strcpy(raw_command,cmd+1);
		strip(raw_command);
		if(raw_command[strlen(raw_command)-1]=='\"')
			raw_command[strlen(raw_command)-1]='\x0';
	        }

	/* else lookup the command */
	else{

		strcpy(temp_buffer,cmd);

		/* get the command name */
		buffer=my_strtok(temp_buffer,"!");

		/* the buffer should never be NULL, but just in case... */
		if(buffer==NULL){
			strcpy(raw_command,"");
			return;
		        }
		else
			strcpy(raw_command,buffer);

		/* clear the previous command arguments */
		for(x=0;x<MAX_COMMAND_ARGUMENTS;x++)
			strcpy(macro_argv[x],"");

		/* get the arguments */
		for(x=0;x<MAX_COMMAND_ARGUMENTS;x++){
			buffer=my_strtok(NULL,"!");
			if(buffer==NULL)
				break;
			strcpy(macro_argv[x],buffer);
			strip(buffer);
		        }

		/* find the command used to check this service */
		temp_command=NULL;
		temp_command=find_command(raw_command,temp_command);

		/* error if we couldn't find the command */
		if(temp_command==NULL)
			return;

		strcpy(raw_command,temp_command->command_line);
		strip(raw_command);
	        }

	/*buffer=&raw_command[0];*/

#ifdef DEBUG1
	printf("\tOutput: %s\n",raw_command);
#endif
#ifdef DEBUG0
	printf("get_raw_command_line() end\n");
#endif

	return;
        }


/* trap signals so we can exit gracefully */
void setup_sighandler(void){

#ifdef DEBUG0
	printf("setup_sighandler() start\n");
#endif

	/* reset the shutdown flag */
	sigshutdown=FALSE;

	/* remove buffering from stderr, stdin, and stdout */
	setbuf(stdin,(char *)NULL);
	setbuf(stdout,(char *)NULL);
	setbuf(stderr,(char *)NULL);

	/* initialize signal handling */
	signal(SIGPIPE,SIG_IGN);
	signal(SIGINT,sighandler);
	signal(SIGQUIT,sighandler);
	signal(SIGTERM,sighandler);
	signal(SIGBUS,sighandler);
	signal(SIGFPE,sighandler);
	signal(SIGILL,sighandler);
	signal(SIGHUP,sighandler);
	signal(SIGABRT,sighandler);	
	signal(SIGSEGV,sighandler);
	signal(SIGKILL,sighandler);

	/* removed child signal handler 08/08/1999 */
	/*signal(SIGCHLD,sighandler);*/

#ifdef DEBUG0
	printf("setup_sighandler() end\n");
#endif

	return;
        }


/* handle signals */
void sighandler(int sig){
	static char *sigs[]={"EXIT","HUP","INT","QUIT","ILL","TRAP","ABRT","BUS","FPE","KILL","USR1","SEGV","USR2","PIPE","ALRM","TERM","STKFLT","CHLD","CONT","STOP","TSTP","TTIN","TTOU","URG","XCPU","XFSZ","VTALRM","PROF","WINCH","IO","PWR","UNUSED","ZERR","DEBUG",(char *)NULL};
	int i;
	char temp_buffer[MAX_INPUT_BUFFER];

	/* if shutdown is already true, we're in a signal trap loop! */
	if(sigshutdown==TRUE)
		exit(ERROR);

	if(sig<0)
		sig=-sig;

	for(i=0;sigs[i]!=(char *)NULL;i++);

	sig%=i;

	if(sig==1){

		sigrestart=TRUE;

		sprintf(temp_buffer,"Caught SIGHUP, restarting...\n");
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);
#ifdef DEBUG2
		printf("%s\n",temp_buffer);
#endif
	        }

	else if(sig<16){

		sigshutdown=TRUE;

		sprintf(temp_buffer,"Caught SIG%s, shutting down...\n",sigs[sig]);
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);
#ifdef DEBUG2
		printf("%s\n",temp_buffer);
#endif
	        }

	return;
        }



/* see if the specified time falls into a valid time range in the given time period */
int check_time_against_period(unsigned long check_time,char *period_name){
	timeperiod *temp_period;
	timerange *temp_range;
	unsigned long midnight_today;
	struct tm *t;

#ifdef DEBUG0
	printf("check_time_against_period() start\n");
#endif
	
	/* if no period name was specified, assume the time is good */
	if(!strcmp(period_name,""))
	   return 1;

	/* if period could not be found, assume the time is good */
	temp_period=find_timeperiod(period_name,NULL);
	if(temp_period==NULL)
		return 1;

	t=localtime((time_t *)&check_time);

	/* calculate the start of the day (midnight, 00:00 hours) */
	t->tm_sec=0;
	t->tm_min=0;
	t->tm_hour=0;
	midnight_today=(unsigned long)mktime(t);

	for(temp_range=temp_period->days[t->tm_wday];temp_range!=NULL;temp_range=temp_range->next){

		/* if the user-specified time falls in this range, return with a positive result */
		if(check_time>=(midnight_today+temp_range->range_start) && check_time<=(midnight_today+temp_range->range_end))
			return 1;
	        }

#ifdef DEBUG0
	printf("check_time_against_period() end\n");
#endif

	return 0;
        }


/* given a preferred time, get the next valid time within a time period */ 
void get_next_valid_time(unsigned long preferred_time,unsigned long *valid_time, char *period_name){
	timeperiod *temp_period;
	timerange *temp_timerange;
	unsigned long midnight_today;
	struct tm *t;
	int day;
	int today;
	int weekday;
	unsigned long earliest_next_valid_time=0L;

#ifdef DEBUG0
	printf("get_next_valid_time() start\n");
#endif

#ifdef DEBUG1
	printf("\tPreferred Time: %lu\n",preferred_time);
#endif

	/* if the preferred time is valid today, go with it */
	if(check_time_against_period(preferred_time,period_name))
		*valid_time=preferred_time;

	/* else find the next available time */
	else{

		/* find the time period - if we can't find it, go with the preferred time */
		temp_period=find_timeperiod(period_name,NULL);
		if(temp_period==NULL){
			*valid_time=preferred_time;
			return;
		        }


		t=localtime((time_t *)&preferred_time);

		/* calculate the start of the day (midnight, 00:00 hours) */
		t->tm_sec=0;
		t->tm_min=0;
		t->tm_hour=0;
		midnight_today=(unsigned long)mktime(t);

		today=t->tm_wday;

		/* check a one week rotation of time */
		for(day=1;day<=7;day++){

			weekday=today+day;
			if(weekday>=7)
				weekday-=7;

			/* check all time ranges for this day of the week */
			for(temp_timerange=temp_period->days[weekday];temp_timerange!=NULL;temp_timerange=temp_timerange->next){

				/* we're looking for the earliest possible start time for this day */
				if(earliest_next_valid_time==0L || temp_timerange->range_start<earliest_next_valid_time)
					earliest_next_valid_time=temp_timerange->range_start;
			        }

			/* break out of the loop if we found a time already */
			if(earliest_next_valid_time!=0L)
				break;
		        }

		/* if we couldn't find a time period (there must be none defined), pass back the preferred time */
		if(earliest_next_valid_time==0L)
			*valid_time=preferred_time;

		/* else we must calculate the number of seconds to use... */
		else
		*valid_time=(unsigned long)(midnight_today+(day*3600*24)+earliest_next_valid_time);
	        }

#ifdef DEBUG1
	printf("\tNext Valid Time: %lu\n",*valid_time);
#endif

#ifdef DEBUG0
	printf("get_next_valid_time() end\n");
#endif

	return;
        }
	
	 

void get_datetime_string(unsigned long *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};

	if(raw_time==NULL)
		time(&t);
	else 
		t=(time_t)(*raw_time);

	tm_ptr=localtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

	snprintf(buffer,buffer_length,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,tzname[1],year);

	return;
        }


/******************************************************************/
/*************** HTML PAGE-CREATION FUNCTIONS *********************/
/******************************************************************/

/* create all HTML pages */
void create_all_html_pages(void){

	if(create_html==TRUE)
		printf("Creating HTML pages...\n\n");

	create_contacts_html_page();
	create_hosts_html_page();
	create_commands_html_page();
	create_services_html_page();
	create_hostgroups_html_page();
	create_contactgroups_html_page();
	create_timeperiods_html_page();

	if(create_html==TRUE)
		printf("\nDone.\n");

	return;
        }


/* create the contact information HTML page */
void create_contacts_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	contact *temp_contact;
	char temp_commands[MAX_COMMANDLINE_LENGTH];
	char *current_command;
	int first_command;
	FILE *fp;
	time_t t;
	int options;
	int odd=0;
	char date_time[48];

	if(create_html==TRUE)
		printf("\tCreating contacts page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_CONTACTS_FILE);
	else
		strcpy(filename,HTML_CONTACTS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Contacts</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Contacts</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Contact Name</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Alias</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Email Address</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Pager</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Notification Options</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host Notification Options</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Notification Period</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host Notification Period</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Notification Commands</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host Notification Commands</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Actions</B></FONT></TD>\n",HTML_CONTACT_BG_COLOR,HTML_CONTACT_FG_COLOR);
	fprintf(fp,"</TR>\n");

	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_CONTACT_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_CONTACT_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_contact->name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_contact->name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_contact->alias);
		fprintf(fp,"<TD valign=\"top\"><A HREF=\"mailto:%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",temp_contact->email,temp_contact->email);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_contact->pager);

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_contact->notify_on_service_critical){
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#CRITICAL\">Critical</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_service_warning){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#WARNING\">Warning</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_service_recovery){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",url_docs_path);
		        }
		if(!options)
			fprintf(fp,"None");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_contact->notify_on_host_down){
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#HOST DOWN\">Down</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_host_unreachable){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#HOST UNREACHABLE\">Unreachable</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_host_recovery){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#HOST RECOVERY\">Recovery</A>",url_docs_path);
		        }
		if(!options)
			fprintf(fp,"None");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD><FONT SIZE=-1>\n");
		if(!strcmp(temp_contact->service_notification_period,""))
			fprintf(fp,"None");
		else
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_TIMEPERIODS_FILE,temp_contact->service_notification_period,temp_contact->service_notification_period);
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD><FONT SIZE=-1>\n");
		if(!strcmp(temp_contact->host_notification_period,""))
			fprintf(fp,"None");
		else
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_TIMEPERIODS_FILE,temp_contact->host_notification_period,temp_contact->host_notification_period);
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		strcpy(temp_commands,temp_contact->commands);
		current_command=my_strtok(temp_commands,",");
		first_command=TRUE;
		while(current_command!=NULL){
			fprintf(fp,"%s<a href=\"%s%s#%s\">%s</a>",(first_command==FALSE)?", ":"",url_html_path,HTML_COMMANDS_FILE,current_command,current_command);
			first_command=FALSE;
			current_command=my_strtok(NULL,",");
		        }
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		strcpy(temp_commands,temp_contact->host_commands);
		current_command=my_strtok(temp_commands,",");
		first_command=TRUE;
		while(current_command!=NULL){
			fprintf(fp,"%s<a href=\"%s%s#%s\">%s</a>",(first_command==FALSE)?", ":"",url_html_path,HTML_COMMANDS_FILE,current_command,current_command);
			first_command=FALSE;
			current_command=my_strtok(NULL,",");
		        }
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"<TD valign=\"top\"><A HREF=\"%s%s?contact=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View All Notifications For This Contact\"></A></TD>\n",url_cgibin_path,NOTIFICATIONS_CGI,temp_contact->name,url_images_path,NOTIFICATION_ICON);
		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }


/* create the host information HTML page */
void create_hosts_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	host *temp_host;
	FILE *fp;
	time_t t;
	int odd=0;
	int options=0;
	char date_time[48];
	int total_seconds=0;
	int hours=0;
	int minutes=0;
	int seconds=0;

	if(create_html==TRUE)
		printf("\tCreating hosts page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_HOSTS_FILE);
	else
		strcpy(filename,HTML_HOSTS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Hosts</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Hosts</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host Name</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Alias/Description</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Address</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);		
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Parent Host</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);		
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Max. Check Attempts</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Interval</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Options</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Period</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host Check Command</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Event Handler</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);	
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Actions</B></FONT></TD>",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);
	fprintf(fp,"</TR>\n");

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_HOST_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_HOST_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_host->name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_host->name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_host->alias);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_host->address);
		if(!strcmp(temp_host->parent_host,""))
			fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>N/A</FONT></TD>\n");
		else
			fprintf(fp,"<TD valign=\"top\"><A HREF=\"%shosts.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_host->parent_host,temp_host->parent_host);
                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%d</FONT></TD>\n",temp_host->max_attempts);

                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		total_seconds=temp_host->notification_interval*interval_length;
		hours=total_seconds/3600;
		total_seconds%=3600;
		minutes=total_seconds/60;
		total_seconds%=60;
		seconds=total_seconds;
		fprintf(fp,"%dh %dm %ds",hours,minutes,seconds);
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_host->notify_on_down){
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#HOST DOWN\">Down</A>",url_docs_path);
		        }
		if(temp_host->notify_on_unreachable){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#HOST UNREACHABLE\">Unreachable</A>",url_docs_path);
		        }
		if(temp_host->notify_on_recovery){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",url_docs_path);
		        }
		if(!options)
			fprintf(fp,"None");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD><FONT SIZE=-1>\n");
		if(!strcmp(temp_host->notification_period,""))
			fprintf(fp,"None");
		else
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_TIMEPERIODS_FILE,temp_host->notification_period,temp_host->notification_period);
		fprintf(fp,"</FONT></TD>\n");

		if(!strcmp(temp_host->host_check_command,""))
			fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>N/A</FONT></TD>\n");
		else
			fprintf(fp,"<TD valign=\"top\"><A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,HTML_COMMANDS_FILE,temp_host->host_check_command,temp_host->host_check_command);
		if(!strcmp(temp_host->event_handler,""))
			fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>N/A</FONT></TD>\n");
		else
			fprintf(fp,"<TD valign=\"top\"><A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,HTML_COMMANDS_FILE,temp_host->event_handler,temp_host->event_handler);
		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A HREF=\"%s%s?host=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Details For This Host\"></A>\n",url_cgibin_path,STATUS_CGI,temp_host->name,url_images_path,STATUS_DETAIL_ICON);   
		fprintf(fp,"<A HREF=\"%s%s?host=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Alert History For This Host\"></A>\n",url_cgibin_path,HISTORY_CGI,temp_host->name,url_images_path,HISTORY_ICON);  
		fprintf(fp,"<A HREF=\"%s%s?host=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View All Notifications For This Host\"></A>\n",url_cgibin_path,NOTIFICATIONS_CGI,temp_host->name,url_images_path,NOTIFICATION_ICON);        
		fprintf(fp,"</TD>\n");
		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }


/* create the command information HTML page */
void create_commands_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	command *temp_command;
	FILE *fp;
	time_t t;
	int odd=0;
	char date_time[48];


	if(create_html==TRUE)
		printf("\tCreating commands page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_COMMANDS_FILE);
	else
		strcpy(filename,HTML_COMMANDS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Commands</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Commands</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Command Name</B></FONT></TD>\n",HTML_COMMAND_BG_COLOR,HTML_COMMAND_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Raw Command Line</B></FONT></TD>\n",HTML_COMMAND_BG_COLOR,HTML_COMMAND_FG_COLOR);
	fprintf(fp,"</TR>\n");

	for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_COMMAND_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_COMMAND_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_command->name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_command->name);
		fprintf(fp,"<TD valign=\"top\"><FONT COLOR=\"%s\" SIZE=-1>%s</FONT></TD>\n",HTML_COMMANDLINE_COLOR,temp_command->command_line);
		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }


/* create the service information HTML page */
void create_services_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	service *temp_service;
	char raw_command[MAX_COMMANDLINE_LENGTH];
	char *buffer;
	FILE *fp;
	time_t t;
	int options;
	int odd=0;
	char *current_contactgroup;
	char temp_contactgroups[MAX_CONTACTGROUPS_LENGTH];
	char date_time[48];
	int total_seconds=0;
	int hours=0;
	int minutes=0;
	int seconds=0;


	if(create_html==TRUE)
		printf("\tCreating services page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_SERVICES_FILE);
	else
		strcpy(filename,HTML_SERVICES_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Services</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Services</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Description</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Check Period</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Normal Check Interval</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Retry Check Interval</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Max. Check Attempts</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Contact Groups</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Interval</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Options</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Period</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Event Handler</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service Check Command</B></FONT></TD>\n",HTML_SERVICE_BG_COLOR,HTML_SERVICE_FG_COLOR);
	fprintf(fp,"</TR>\n");

	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_SERVICE_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_SERVICE_BG_EVEN_COLOR);
		        }
		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s;%s\"></A>\n",temp_service->host_name,temp_service->description);
		fprintf(fp,"<A HREF=\"%shosts.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,temp_service->host_name,temp_service->host_name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_service->description);

		fprintf(fp,"<TD><FONT SIZE=-1>\n");
		if(!strcmp(temp_service->check_period,""))
			fprintf(fp,"None");
		else
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_TIMEPERIODS_FILE,temp_service->check_period,temp_service->check_period);
		fprintf(fp,"</FONT></TD>\n");

                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		total_seconds=temp_service->check_interval*interval_length;
		hours=total_seconds/3600;
		total_seconds%=3600;
		minutes=total_seconds/60;
		total_seconds%=60;
		seconds=total_seconds;
		fprintf(fp,"%dh %dm %ds",hours,minutes,seconds);
		fprintf(fp,"</FONT></TD>\n");

                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		total_seconds=temp_service->retry_interval*interval_length;
		hours=total_seconds/3600;
		total_seconds%=3600;
		minutes=total_seconds/60;
		total_seconds%=60;
		seconds=total_seconds;
		fprintf(fp,"%dh %dm %ds",hours,minutes,seconds);
		fprintf(fp,"</FONT></TD>\n");

                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%d</FONT></TD>\n",temp_service->max_attempts);

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		strcpy(temp_contactgroups,temp_service->contact_groups);
		current_contactgroup=my_strtok(temp_contactgroups,",");
		options=0;
		while(current_contactgroup!=NULL){
			if(options)
				fprintf(fp,", ");
			fprintf(fp,"<A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A>\n",url_html_path,HTML_CONTACTGROUPS_FILE,current_contactgroup,current_contactgroup);
			options=1;
			current_contactgroup=my_strtok(NULL,",");
		        }
		fprintf(fp,"</FONT></TD>\n");


                fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		total_seconds=temp_service->notification_interval*interval_length;
		hours=total_seconds/3600;
		total_seconds%=3600;
		minutes=total_seconds/60;
		total_seconds%=60;
		seconds=total_seconds;
		fprintf(fp,"%dh %dm %ds",hours,minutes,seconds);
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_service->notify_on_critical){
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#CRITICAL\">Critical</A>",url_docs_path);
		        }
		if(temp_service->notify_on_warning){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#WARNING\">Warning</A>",url_docs_path);
		        }
		if(temp_service->notify_on_recovery){
			fprintf(fp,"%s",(options)?", ":"");
			options=1;
			fprintf(fp,"<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",url_docs_path);
		        }
		if(!options)
			fprintf(fp,"None");
		fprintf(fp,"</FONT></TD>\n");

		fprintf(fp,"<TD><FONT SIZE=-1>\n");
		if(!strcmp(temp_service->notification_period,""))
			fprintf(fp,"None");
		else
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_TIMEPERIODS_FILE,temp_service->notification_period,temp_service->notification_period);
		fprintf(fp,"</FONT></TD>\n");

		if(!strcmp(temp_service->event_handler,""))
			fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>N/A</FONT></TD>\n");
		else
			fprintf(fp,"<TD valign=\"top\"><A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,HTML_COMMANDS_FILE,temp_service->event_handler,temp_service->event_handler);
		if(temp_service->command[0]=='\"')
			fprintf(fp,"<TD valign=\"top\"><I><FONT SIZE=-1>%s</FONT></I></TD>\n",temp_service->command);
		else{
			strcpy(raw_command,temp_service->command);
			buffer=strtok(raw_command,"!");
			fprintf(fp,"<TD valign=\"top\"><A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",url_html_path,HTML_COMMANDS_FILE,buffer,temp_service->command);
		        }

		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }


/* create the host group information HTML page */
void create_hostgroups_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	hostgroup *temp_hostgroup;
	hostgroupmember *temp_member;
	int first_contactgroup;
	char *current_contactgroup;
	char temp_contactgroups[MAX_CONTACTGROUPS_LENGTH];
	FILE *fp;
	time_t t;
	int odd=0;
	char date_time[48];

	if(create_html==TRUE)
		printf("\tCreating hostgroups page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_HOSTGROUPS_FILE);
	else
		strcpy(filename,HTML_HOSTGROUPS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Host Groups</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Host Groups</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Group Name</B></FONT></TD>\n",HTML_HOSTGROUP_BG_COLOR,HTML_HOSTGROUP_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Description</B></FONT></TD>\n",HTML_HOSTGROUP_BG_COLOR,HTML_HOSTGROUP_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Contact Groups</B></FONT></TD>\n",HTML_HOSTGROUP_BG_COLOR,HTML_HOSTGROUP_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Hosts</B></FONT></TD>\n",HTML_HOSTGROUP_BG_COLOR,HTML_HOSTGROUP_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Actions</B></FONT></TD>\n",HTML_HOST_BG_COLOR,HTML_HOST_FG_COLOR);
	fprintf(fp,"</TR>\n");


	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_HOSTGROUP_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_HOSTGROUP_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_hostgroup->group_name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_hostgroup->group_name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_hostgroup->alias);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");
		strcpy(temp_contactgroups,temp_hostgroup->contact_groups);
		current_contactgroup=my_strtok(temp_contactgroups,",");
		first_contactgroup=TRUE;
		while(current_contactgroup!=NULL){
			fprintf(fp,"%s<A HREF=\"%s%s#%s\"><FONT SIZE=-1>%s</FONT></A>\n",(first_contactgroup==TRUE)?"":", ",url_html_path,HTML_CONTACTGROUPS_FILE,current_contactgroup,current_contactgroup);
			first_contactgroup=FALSE;
			current_contactgroup=my_strtok(NULL,",");
		        }
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");

		/* find all the group members */
		for(temp_member=temp_hostgroup->members;temp_member!=NULL;temp_member=temp_member->next){

			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_HOSTS_FILE,temp_member->host_name,temp_member->host_name);

			if(temp_member->next!=NULL)
				fprintf(fp,", ");
		        }

		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"<TD>");
		fprintf(fp,"<A HREF=\"%s%s?hostgroup=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Overview For This Host Group\"></A>\n",url_cgibin_path,STATUS_CGI,temp_hostgroup->group_name,url_images_path,STATUS_OVERVIEW_ICON);
		fprintf(fp,"<A HREF=\"%s%s?hostgroup=%s&style=detail\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Detail For This Host Group\"></A>\n",url_cgibin_path,STATUS_CGI,temp_hostgroup->group_name,url_images_path,STATUS_DETAIL_ICON);
		fprintf(fp,"</TD>");
		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }


/* create contact group information HTML page */
void create_contactgroups_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_member;
	FILE *fp;
	time_t t;
	int odd=0;
	char date_time[48];

	if(create_html==TRUE)
		printf("\tCreating contactgroups page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_CONTACTGROUPS_FILE);
	else
		strcpy(filename,HTML_CONTACTGROUPS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Contact Groups</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Contact Groups</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLPADDING=3 CELLSPACING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD valign=\"top\" bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Group Name</B></FONT></TD>\n",HTML_CONTACTGROUP_BG_COLOR,HTML_CONTACTGROUP_FG_COLOR);
	fprintf(fp,"<TD valign=\"top\" bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Description</B></FONT></TD>\n",HTML_CONTACTGROUP_BG_COLOR,HTML_CONTACTGROUP_FG_COLOR);
	fprintf(fp,"<TD valign=\"top\" bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Group Members</B></FONT></TD>\n",HTML_CONTACTGROUP_BG_COLOR,HTML_CONTACTGROUP_FG_COLOR);
	fprintf(fp,"</TR>\n");

	/* find all the contact groups */
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_CONTACTGROUP_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_CONTACTGROUP_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_contactgroup->group_name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_contactgroup->group_name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_contactgroup->alias);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>");

		/* find all the group members */
		for(temp_member=temp_contactgroup->members;temp_member!=NULL;temp_member=temp_member->next){
			fprintf(fp,"<A HREF=\"%s%s#%s\">%s</A>\n",url_html_path,HTML_CONTACTS_FILE,temp_member->contact_name,temp_member->contact_name);
			if(temp_member->next!=NULL)
				fprintf(fp,", ");
		        }
		fprintf(fp,"</FONT></TD>\n");
		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);


	return;
        }



/* create the timeperiods information HTML page */
void create_timeperiods_html_page(void){
	char filename[MAX_FILENAME_LENGTH];
	timeperiod *temp_timeperiod;
	FILE *fp;
	time_t t;
	int odd=0;
	int day;
	int hours;
	int minutes;
	int total_seconds;
	int option;
	timerange *temp_timerange;
	char date_time[48];

	if(create_html==TRUE)
		printf("\tCreating timeperiods page...\n");

	/* get the filename we should use */
	if(create_html==FALSE)
		sprintf(filename,"%s%s",physical_html_path,HTML_TIMEPERIODS_FILE);
	else
		strcpy(filename,HTML_TIMEPERIODS_FILE);

	fp=fopen(filename,"w");
	if(fp==NULL){
		printf("\tError: Cannot open file '%s' for writing!\n",filename);
		return;
	        }

	fprintf(fp,"<HTML>\n");
	fprintf(fp,"<HEAD>\n");
	fprintf(fp,"<TITLE>NetSaint Time Periods</TITLE>\n");
	fprintf(fp,"</HEAD>\n");
	fprintf(fp,"<BODY>\n");

	fprintf(fp,"<P><CENTER>\n");
	fprintf(fp,"<H2>Time Periods</H2>\n");
	time(&t);
	get_datetime_string(&t,date_time,sizeof(date_time));
	fprintf(fp,"<FONT SIZE=-1>Last Updated: %s</FONT>",date_time);
	fprintf(fp,"</CENTER></P>\n");

	fprintf(fp,"<P>\n");
	fprintf(fp,"<CENTER>\n");
	fprintf(fp,"<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=3>\n");
	fprintf(fp,"<TR>\n");
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Time Period Name</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Alias/Description</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Sunday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Monday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Tuesday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Wednesday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Thursday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Friday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"<TD bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Saturday Time Ranges</B></FONT></TD>\n",HTML_TIMEPERIOD_BG_COLOR,HTML_TIMEPERIOD_FG_COLOR);
	fprintf(fp,"</TR>\n");

	for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){

		if(odd){
			odd=0;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_TIMEPERIOD_BG_ODD_COLOR);
		        }
		else{
			odd=1;
			fprintf(fp,"<TR bgcolor=%s>\n",HTML_TIMEPERIOD_BG_EVEN_COLOR);
		        }

		fprintf(fp,"<TD valign=\"top\">");
		fprintf(fp,"<A NAME=\"%s\"></A>\n",temp_timeperiod->name);
		fprintf(fp,"<FONT SIZE=-1>%s</FONT></TD>\n",temp_timeperiod->name);
		fprintf(fp,"<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_timeperiod->alias);

		for(day=0;day<7;day++){

			fprintf(fp,"<TD><FONT SIZE=-1>");
			option=0;

			if(temp_timeperiod->days[day]==NULL)
				fprintf(fp,"None");

			else{
				for(temp_timerange=temp_timeperiod->days[day];temp_timerange!=NULL;temp_timerange=temp_timerange->next){

					if(option)
						fprintf(fp,", ");

					total_seconds=temp_timerange->range_start;
					hours=(int)(total_seconds/3600);
					minutes=(int)((total_seconds-(hours*3600))/60);
					fprintf(fp,"%02d:%02d-",hours,minutes);

					total_seconds=temp_timerange->range_end;
					hours=(int)(total_seconds/3600);
					minutes=(int)((total_seconds-(hours*3600))/60);
					fprintf(fp,"%d:%02d",hours,minutes);
				
					option=1;
			                }
			        }

			fprintf(fp,"</FONT></TD>\n");
		        }

		fprintf(fp,"</TR>\n");
	        }

	fprintf(fp,"</TABLE>\n");
	fprintf(fp,"</CENTER>\n");
	fprintf(fp,"</P>\n");

	fprintf(fp,"</BODY>\n");
	fprintf(fp,"</HTML>\n");

	fclose(fp);

	return;
        }



/******************************************************************/
/*********************** CLEANUP FUNCTIONS ************************/
/******************************************************************/

/* do some cleanup before we exit */
void cleanup(void){

#ifdef DEBUG0
	printf("cleanup() start\n");
#endif

	free_memory();
	reset_variables();
	clear_volatile_macros();
	clear_nonvolatile_macros();

#ifdef DEBUG0
	printf("cleanup() end\n");
#endif

	return;
	}


/* free the memory allocated to the linked lists */
void free_memory(void){
	timeperiod *this_timeperiod=NULL;
	timeperiod *next_timeperiod=NULL;
	timerange *this_timerange=NULL;
	timerange *next_timerange=NULL;
	host *this_host=NULL;
	host *next_host=NULL;
	hostgroup *this_hostgroup=NULL;
	hostgroup *next_hostgroup=NULL;
	hostgroupmember *this_hostgroupmember=NULL;
	hostgroupmember *next_hostgroupmember=NULL;
	contact	*this_contact=NULL;
	contact *next_contact=NULL;
	contactgroup *this_contactgroup=NULL;
	contactgroup *next_contactgroup=NULL;
	contactgroupmember *this_contactgroupmember=NULL;
	contactgroupmember *next_contactgroupmember=NULL;
	service *this_service=NULL;
	service *next_service=NULL;
	timed_event *this_event=NULL;
	timed_event *next_event=NULL;
	command *this_command=NULL;
	command *next_command=NULL;
	int day;

#ifdef DEBUG0
	printf("free_memory() start\n");
#endif

	/* free memory for the timeperiod list */
	this_timeperiod=timeperiod_list;
	while(this_timeperiod!=NULL){

		/* free the time ranges contained in this timeperiod */
		for(day=0;day<7;day++){

			for(this_timerange=this_timeperiod->days[day];this_timerange!=NULL;this_timerange=next_timerange){
				next_timerange=this_timerange->next;
				free(this_timerange);
			        }
		        }

		next_timeperiod=this_timeperiod->next;
		free(this_timeperiod);
		this_timeperiod=next_timeperiod;
		}

	/* reset the host pointer */
	timeperiod_list=NULL;

#ifdef DEBUG1
	printf("\timeperiod_list freed\n");
#endif

	/* free memory for the host list */
	this_host=host_list;
	while(this_host!=NULL){
		next_host=this_host->next;
		free(this_host);
		this_host=next_host;
		}

	/* reset the host pointer */
	host_list=NULL;

#ifdef DEBUG1
	printf("\thost_list freed\n");
#endif

	/* free memory for the host group list */
	this_hostgroup=hostgroup_list;
	while(this_hostgroup!=NULL){

		/* free memory for the group members */
		this_hostgroupmember=this_hostgroup->members;
		while(this_hostgroupmember!=NULL){
			next_hostgroupmember=this_hostgroupmember->next;
			free(this_hostgroupmember);
			this_hostgroupmember=next_hostgroupmember;
		        }

		next_hostgroup=this_hostgroup->next;
		free(this_hostgroup);
		this_hostgroup=next_hostgroup;
		}

	/* reset the hostgroup pointer */
	hostgroup_list=NULL;

#ifdef DEBUG1
	printf("\thostgroup_list freed\n");
#endif

	/* free memory for the contact list */
	this_contact=contact_list;
	while(this_contact!=NULL){
		next_contact=this_contact->next;
		free(this_contact);
		this_contact=next_contact;
		}

	/* reset the contact pointer */
	contact_list=NULL;

#ifdef DEBUG1
	printf("\tcontact_list freed\n");
#endif

	/* free memory for the contact group list */
	this_contactgroup=contactgroup_list;
	while(this_contactgroup!=NULL){

		/* free memory for the group members */
		this_contactgroupmember=this_contactgroup->members;
		while(this_contactgroupmember!=NULL){
			next_contactgroupmember=this_contactgroupmember->next;
			free(this_contactgroupmember);
			this_contactgroupmember=next_contactgroupmember;
		        }

		next_contactgroup=this_contactgroup->next;
		free(this_contactgroup);
		this_contactgroup=next_contactgroup;
		}

	/* reset the contactgroup pointer */
	contactgroup_list=NULL;

#ifdef DEBUG1
	printf("\tcontactgroup_list freed\n");
#endif

	/* free memory for the service list */
	this_service=service_list;
	while(this_service!=NULL){
		next_service=this_service->next;
		free(this_service);
		this_service=next_service;
		}

	/* reset the service pointer */
	service_list=NULL;

#ifdef DEBUG1
	printf("\tservice_list freed\n");
#endif

	/* free memory for the command list */
	this_command=command_list;
	while(this_command!=NULL){
		next_command=this_command->next;
		free(this_command);
		this_command=next_command;
	        }

	/* reset the command list */
	command_list=NULL;

#ifdef DEBUG1
	printf("\tcommand_list freed\n");
#endif

	/* free memory for the event list */
	this_event=event_list;
	while(this_event!=NULL){
		next_event=this_event->next;
		free(this_event);
		this_event=next_event;
	        }

	/* reset the event pointer */
	event_list=NULL;

#ifdef DEBUG1
	printf("\tevent_list freed\n");
#endif

	/* free memory used by my_strtok() function */
	if(original_my_strtok_buffer!=NULL)
		free(original_my_strtok_buffer);

	/* reset the my_strtok() buffers */
	original_my_strtok_buffer=NULL;
	my_strtok_buffer=NULL;

#ifdef DEBUG1
	printf("\tmy_strtok() buffers freed\n");
#endif

	/* free any notification list that may have been overlooked */
	free_notification_list();

#ifdef DEBUG1
	printf("\tnotification_list freed\n");
#endif

#ifdef DEBUG0
	printf("free_memory() end\n");
#endif

	return;
	}


/* free a notification list that was created */
void free_notification_list(void){
	notification *temp_notification;
	notification *next_notification;

#ifdef DEBUG0
	printf("free_notification_list() start\n");
#endif

	temp_notification=notification_list;
	while(temp_notification!=NULL){
		next_notification=temp_notification->next;
		free((void *)temp_notification);
		temp_notification=next_notification;
	        }

	/* reset notification list pointer */
	notification_list=NULL;

#ifdef DEBUG0
	printf("free_notification_list() end\n");
#endif

	return;
        }


/* reset all system-wide variables, so when we've receive a SIGHUP we can restart cleanly */
int reset_variables(void){

#ifdef DEBUG0
	printf("reset_variables() start\n");
#endif

	strcpy(hosts_config_file,"");
	strcpy(log_file,DEFAULT_LOG_FILE);
	strcpy(temp_file,DEFAULT_TEMP_FILE);
	strcpy(status_file,DEFAULT_STATUS_FILE);

	strcpy(physical_html_path,DEFAULT_PHYSICAL_HTML_PATH);
	strcpy(url_html_path,DEFAULT_URL_HTML_PATH);
	sprintf(url_images_path,"%simages/",DEFAULT_URL_HTML_PATH);
	sprintf(url_docs_path,"%sdocs/",DEFAULT_URL_HTML_PATH);
	strcpy(url_cgibin_path,DEFAULT_URL_CGIBIN_PATH);

	strcpy(generic_summary,"");
	
	my_strtok_buffer=NULL;
	original_my_strtok_buffer=NULL;

	log_level=DEFAULT_LOG_LEVEL;
	use_syslog=DEFAULT_USE_SYSLOG;
	syslog_level=DEFAULT_SYSLOG_LEVEL;
	log_notifications=DEFAULT_NOTIFICATION_LOGGING;
	auto_generate_html=DEFAULT_AUTO_GENERATE_HTML;
	log_service_retries=DEFAULT_LOG_SERVICE_RETRIES;
	log_host_retries=DEFAULT_LOG_HOST_RETRIES;
	log_event_handlers=DEFAULT_LOG_EVENT_HANDLERS;

	sleep_time=DEFAULT_SLEEP_TIME;
	interval_length=DEFAULT_INTERVAL_LENGTH;
	inter_check_delay=DEFAULT_INTER_CHECK_DELAY;

	max_child_process_time=DEFAULT_MAX_CHILD_PROCESS_TIME;

	use_agressive_host_checking=DEFAULT_AGRESSIVE_HOST_CHECKING;

	contact_list=NULL;
	contactgroup_list=NULL;
	host_list=NULL;
	hostgroup_list=NULL;
	service_list=NULL;
	event_list=NULL;
	notification_list=NULL;
	command_list=NULL;
	timeperiod_list=NULL;


#ifdef DEBUG0
	printf("reset_variables() end\n");
#endif

	return OK;
        }


/* attempts to clean up orphaned child processes that got left behind */
int clear_zombies(void){
	int cleared_zombies=0;
	int current_orphan;
	char temp_buffer[MAX_INPUT_BUFFER];
	pid_t wait_result;
	int status;

#ifdef DEBUG0
	printf("clear_zombies() start\n");
#endif

#ifdef DEBUG1
	printf("\tTotal orphaned children: %d\n",total_orphaned_children);
#endif

	/* try to free all orphaned child processes */
	for(current_orphan=1;current_orphan<=total_orphaned_children;current_orphan++){

		/* send SIGKILL and SIGTERM signals to all child processes */
		kill(0,SIGTERM);
		kill(0,SIGKILL);

		/* try to catch a child process that has exited, but don't hang around */
		wait_result=waitpid(-1,&status,WNOHANG | WUNTRACED);

		/* there weren't any child processes that have exited, so break from the loop */
		if(wait_result==0)
			break;

		/* we freed a zombie! */
		else if(wait_result>0){

			cleared_zombies++;

			sprintf(temp_buffer,"We killed a zombie process [%d]!",wait_result);
			write_to_log(temp_buffer);
		       }
	        }

	total_orphaned_children-=cleared_zombies;

#ifdef DEBUG1
	printf("\tTotal cleared zombies: %d\n",cleared_zombies);
#endif

#ifdef DEBUG0
	printf("clear_zombies() end\n");
#endif
	return cleared_zombies;
        }

